# Copyright 2016 Radware LTD.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.lb.v2 import l7rule
from neutronclient.tests.unit import test_cli20


"""Structure for mapping cli and api arguments

The structure maps cli arguments and a list of its
api argument name, default cli value and default api value.
It helps to make tests more general for different argument types.
"""
args_conf = {
    'admin-state-up': ['admin_state_up', True, True],
    'admin-state-down': ['admin_state_up', None, False],
    'type': ['type', 'HOST_NAME', 'HOST_NAME'],
    'compare-type': ['compare_type', 'EQUAL_TO', 'EQUAL_TO'],
    'invert-compare': ['invert', None, True],
    'key': ['key', 'key', 'key'],
    'value': ['value', 'value', 'value']}


class CLITestV20LbL7RuleJSON(test_cli20.CLITestV20Base):

    def _get_test_args(self, *args, **kwargs):
        """Function for generically building testing arguments"""
        cli_args = []
        api_args = {}
        for arg in args:
            cli_args.append('--' + arg)
            if not args_conf[arg][1]:
                pass
            elif arg in kwargs:
                cli_args.append(str(kwargs[arg]))
            else:
                cli_args.append(args_conf[arg][1])

            if arg in kwargs:
                api_args[args_conf[arg][0]] = kwargs[arg]
            else:
                api_args[args_conf[arg][0]] = args_conf[arg][2]

        if 'invert' not in api_args:
            api_args['invert'] = False
        return cli_args, api_args

    def _test_create_rule(self, *args, **kwargs):
        resource = 'rule'
        cmd_resource = 'lbaas_l7rule'
        cmd = l7rule.CreateL7Rule(test_cli20.MyApp(sys.stdout), None)
        cli_args, api_args = self._get_test_args(*args, **kwargs)
        position_names = list(api_args.keys())
        position_values = list(api_args.values())
        cli_args.append('test_policy')
        self._test_create_resource(resource, cmd, None, 'test_id',
                                   cli_args, position_names, position_values,
                                   cmd_resource=cmd_resource,
                                   parent_id='test_policy')

    def _test_update_rule(self, *args, **kwargs):
        resource = 'rule'
        cmd_resource = 'lbaas_l7rule'
        cmd = l7rule.UpdateL7Rule(test_cli20.MyApp(sys.stdout), None)
        cli_args, api_args = self._get_test_args(*args, **kwargs)
        cli_args.append('test_id')
        cli_args.append('test_policy')
        self._test_update_resource(resource, cmd, 'test_id',
                                   cli_args, api_args,
                                   cmd_resource=cmd_resource,
                                   parent_id='test_policy')

    def test_create_rule_with_mandatory_params(self):
        # lbaas-l7rule-create with mandatory params only.

        self._test_create_rule('type', 'compare-type',
                               'value')

    def test_create_disabled_rule(self):
        # lbaas-l7rule-create disabled rule.

        self._test_create_rule('type', 'compare-type',
                               'value', 'admin-state-down')

    def test_create_rule_with_all_params(self):
        # lbaas-l7rule-create with all params set.

        self._test_create_rule('type', 'compare-type',
                               'invert-compare', 'key', 'value',
                               type='HEADER', compare_type='CONTAINS',
                               key='other_key', value='other_value')

    def test_create_rule_with_inverted_compare(self):
        # lbaas-l7rule-create with invertted compare type.

        self._test_create_rule('type', 'compare-type',
                               'invert-compare', 'value')

    def test_list_rules(self):
        # lbaas-l7rule-list.

        resources = 'rules'
        cmd_resources = 'lbaas_l7rules'
        cmd = l7rule.ListL7Rule(test_cli20.MyApp(sys.stdout), None)

        policy_id = 'policy_id'
        self._test_list_resources(resources, cmd, True,
                                  base_args=[policy_id],
                                  cmd_resources=cmd_resources,
                                  parent_id=policy_id,
                                  query="l7policy_id=%s" % policy_id)

    def test_list_rules_pagination(self):
        # lbaas-l7rule-list with pagination.

        resources = 'rules'
        cmd_resources = 'lbaas_l7rules'
        cmd = l7rule.ListL7Rule(test_cli20.MyApp(sys.stdout), None)
        policy_id = 'policy_id'
        self._test_list_resources_with_pagination(
            resources, cmd, base_args=[policy_id],
            cmd_resources=cmd_resources, parent_id=policy_id,
            query="l7policy_id=%s" % policy_id)

    def test_list_rules_sort(self):
        # lbaas-l7rule-list --sort-key id --sort-key asc.

        resources = 'rules'
        cmd_resources = 'lbaas_l7rules'
        cmd = l7rule.ListL7Rule(test_cli20.MyApp(sys.stdout), None)
        policy_id = 'policy_id'
        self._test_list_resources(
            resources, cmd, True, base_args=[policy_id],
            cmd_resources=cmd_resources, parent_id=policy_id,
            query="l7policy_id=%s" % policy_id)

    def test_list_rules_limit(self):
        # lbaas-l7rule-list -P.

        resources = 'rules'
        cmd_resources = 'lbaas_l7rules'
        cmd = l7rule.ListL7Rule(test_cli20.MyApp(sys.stdout), None)
        policy_id = 'policy_id'
        self._test_list_resources(resources, cmd, page_size=1000,
                                  base_args=[policy_id],
                                  cmd_resources=cmd_resources,
                                  parent_id=policy_id,
                                  query="l7policy_id=%s" % policy_id)

    def test_show_rule_id(self):
        # lbaas-l7rule-show test_id.

        resource = 'rule'
        cmd_resource = 'lbaas_l7rule'
        policy_id = 'policy_id'
        cmd = l7rule.ShowL7Rule(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id, policy_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'],
                                 cmd_resource=cmd_resource,
                                 parent_id=policy_id)

    def test_update_rule_type(self):
        # lbaas-l7rule-update test_id --type HEADER test_policy

        self._test_update_rule('type', type='HEADER')

    def test_update_rule_compare_type(self):
        # lbaas-l7rule-update test_id --compare-type CONTAINS test_policy.

        self._test_update_rule('compare-type',
                               **{'compare-type': 'CONTAINS'})

    def test_update_rule_inverted_compare_type(self):
        # lbaas-l7rule-update test_id --invert-compare test_policy.

        self._test_update_rule('invert-compare')

    def test_update_rule_key_value(self):
        # lbaas-l7rule-update test_id --key other --value other test_policy.

        self._test_update_rule('key', 'value',
                               key='other', value='other')

    def test_delete_rule(self):
        # lbaas-l7rule-delete test_id policy_id.

        resource = 'rule'
        cmd_resource = 'lbaas_l7rule'
        policy_id = 'policy_id'
        test_id = 'test_id'
        cmd = l7rule.DeleteL7Rule(test_cli20.MyApp(sys.stdout), None)
        args = [test_id, policy_id]
        self._test_delete_resource(resource, cmd, test_id, args,
                                   cmd_resource=cmd_resource,
                                   parent_id=policy_id)



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import itertools
import sys
from unittest import mock


from neutronclient.neutron.v2_0 import port
from neutronclient import shell
from neutronclient.tests.unit import test_cli20


class CLITestV20PortJSON(test_cli20.CLITestV20Base):
    def setUp(self):
        super(CLITestV20PortJSON, self).setUp(plurals={'tags': 'tag'})

    def test_create_port(self):
        # Create port: netid.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = [netid, '--description', 'DESC']
        position_names = ['network_id']
        position_values = []
        position_values.extend([netid])
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   description='DESC')

    def test_create_port_extra_dhcp_opts_args(self):
        # Create port: netid --extra_dhcp_opt.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        extra_dhcp_opts = [{'opt_name': 'bootfile-name',
                            'opt_value': 'pxelinux.0'},
                           {'opt_name': 'tftp-server',
                            'opt_value': '123.123.123.123'},
                           {'opt_name': 'server-ip-address',
                            'opt_value': '123.123.123.45'}]
        args = [netid]
        for dhcp_opt in extra_dhcp_opts:
            args += ['--extra-dhcp-opt',
                     ('opt_name=%(opt_name)s,opt_value=%(opt_value)s' %
                      dhcp_opt)]
        position_names = ['network_id', 'extra_dhcp_opts']
        position_values = [netid, extra_dhcp_opts]
        position_values.extend([netid])
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_extra_dhcp_opts_args_ip_version(self):
        # Create port: netid --extra_dhcp_opt.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        extra_dhcp_opts = [{'opt_name': 'bootfile-name',
                            'opt_value': 'pxelinux.0',
                            'ip_version': "4"},
                           {'opt_name': 'tftp-server',
                            'opt_value': '2001:192:168::1',
                            'ip_version': "6"},
                           {'opt_name': 'server-ip-address',
                            'opt_value': '123.123.123.45',
                            'ip_version': "4"}]
        args = [netid]
        for dhcp_opt in extra_dhcp_opts:
            args += ['--extra-dhcp-opt',
                     ('opt_name=%(opt_name)s,opt_value=%(opt_value)s,'
                      'ip_version=%(ip_version)s' %
                      dhcp_opt)]
        position_names = ['network_id', 'extra_dhcp_opts']
        position_values = [netid, extra_dhcp_opts]
        position_values.extend([netid])
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_full(self):
        # Create port: --mac_address mac --device_id deviceid netid.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--mac_address', 'mac', '--device_id', 'deviceid', netid]
        position_names = ['network_id', 'mac_address', 'device_id']
        position_values = [netid, 'mac', 'deviceid']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

        # Test dashed options
        args = ['--mac-address', 'mac', '--device-id', 'deviceid', netid]
        position_names = ['network_id', 'mac_address', 'device_id']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_vnic_type_normal(self):
        # Create port: --vnic_type normal netid.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--vnic_type', 'normal', netid]
        position_names = ['binding:vnic_type', 'network_id']
        position_values = ['normal', netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

        # Test dashed options
        args = ['--vnic-type', 'normal', netid]
        position_names = ['binding:vnic_type', 'network_id']
        position_values = ['normal', netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_vnic_type_direct(self):
        # Create port: --vnic_type direct netid.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--vnic_type', 'direct', netid]
        position_names = ['binding:vnic_type', 'network_id']
        position_values = ['direct', netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

        # Test dashed options
        args = ['--vnic-type', 'direct', netid]
        position_names = ['binding:vnic_type', 'network_id']
        position_values = ['direct', netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_vnic_type_direct_physical(self):
        # Create port: --vnic_type direct-physical netid.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--vnic_type', 'direct-physical', netid]
        position_names = ['binding:vnic_type', 'network_id']
        position_values = ['direct-physical', netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

        # Test dashed options
        args = ['--vnic-type', 'direct-physical', netid]
        position_names = ['binding:vnic_type', 'network_id']
        position_values = ['direct-physical', netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_vnic_type_macvtap(self):
        # Create port: --vnic_type macvtap netid.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--vnic_type', 'macvtap', netid]
        position_names = ['binding:vnic_type', 'network_id']
        position_values = ['macvtap', netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

        # Test dashed options
        args = ['--vnic-type', 'macvtap', netid]
        position_names = ['binding:vnic_type', 'network_id']
        position_values = ['macvtap', netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_vnic_type_baremetal(self):
        # Create port: --vnic_type baremetal netid.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--vnic_type', 'baremetal', netid]
        position_names = ['binding:vnic_type', 'network_id']
        position_values = ['baremetal', netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

        # Test dashed options
        args = ['--vnic-type', 'baremetal', netid]
        position_names = ['binding:vnic_type', 'network_id']
        position_values = ['baremetal', netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_with_binding_profile(self):
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--binding_profile', '{"foo":"bar"}', netid]
        position_names = ['binding:profile', 'network_id']
        position_values = [{'foo': 'bar'}, netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

        # Test dashed options
        args = ['--binding-profile', '{"foo":"bar"}', netid]
        position_names = ['binding:profile', 'network_id']
        position_values = [{'foo': 'bar'}, netid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_tenant(self):
        # Create port: --tenant_id tenantid netid.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--tenant_id', 'tenantid', netid, ]
        position_names = ['network_id']
        position_values = []
        position_values.extend([netid])
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

        # Test dashed options
        args = ['--tenant-id', 'tenantid', netid, ]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_port_tags(self):
        # Create port: netid mac_address device_id --tags a b.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = [netid, '--tags', 'a', 'b']
        position_names = ['network_id']
        position_values = []
        position_values.extend([netid])
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tags=['a', 'b'])

    def test_create_port_secgroup(self):
        # Create port: --security-group sg1_id netid.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--security-group', 'sg1_id', netid]
        position_names = ['network_id', 'security_groups']
        position_values = [netid, ['sg1_id']]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_secgroups(self):
        # Create port: <security_groups> netid
        # The <security_groups> are --security-group sg1_id
        # --security-group sg2_id
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--security-group', 'sg1_id',
                '--security-group', 'sg2_id',
                netid]
        position_names = ['network_id', 'security_groups']
        position_values = [netid, ['sg1_id', 'sg2_id']]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_secgroup_off(self):
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--no-security-groups', netid]
        position_names = ['network_id', 'security_groups']
        position_values = [netid, []]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_secgroups_list(self):
        # Create port: netid <security_groups>
        # The <security_groups> are --security-groups list=true sg_id1 sg_id2
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = [netid, '--security-groups', 'list=true', 'sg_id1', 'sg_id2']
        position_names = ['network_id', 'security_groups']
        position_values = [netid, ['sg_id1', 'sg_id2']]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_with_qos_policy(self):
        # Create port: --qos-policy mypolicy.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        qos_policy_name = 'mypolicy'
        args = [netid, '--qos-policy', qos_policy_name]
        position_names = ['network_id', 'qos_policy_id']
        position_values = [netid, qos_policy_name]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_with_dns_name(self):
        # Create port: --dns-name my-port.
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        dns_name_name = 'my-port'
        args = [netid, '--dns-name', dns_name_name]
        position_names = ['network_id', 'dns_name']
        position_values = [netid, dns_name_name]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_with_allowed_address_pair_ipaddr(self):
        # Create port:
        # --allowed-address-pair ip_address=addr0
        # --allowed-address-pair ip_address=addr1
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        pairs = [{'ip_address': '123.123.123.123'},
                 {'ip_address': '123.123.123.45'}]
        args = [netid,
                '--allowed-address-pair',
                'ip_address=123.123.123.123',
                '--allowed-address-pair',
                'ip_address=123.123.123.45']
        position_names = ['network_id', 'allowed_address_pairs']
        position_values = [netid, pairs]
        position_values.extend([netid])
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_port_with_allowed_address_pair(self):
        # Create port:
        # --allowed-address-pair ip_address=addr0,mac_address=mac0
        # --allowed-address-pair ip_address=addr1,mac_address=mac1
        resource = 'port'
        cmd = port.CreatePort(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        pairs = [{'ip_address': '123.123.123.123',
                  'mac_address': '10:00:00:00:00:00'},
                 {'ip_address': '123.123.123.45',
                  'mac_address': '10:00:00:00:00:01'}]
        args = [netid,
                '--allowed-address-pair',
                'ip_address=123.123.123.123,mac_address=10:00:00:00:00:00',
                '--allowed-address-pair',
                'ip_address=123.123.123.45,mac_address=10:00:00:00:00:01']
        position_names = ['network_id', 'allowed_address_pairs']
        position_values = [netid, pairs]
        position_values.extend([netid])
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_list_ports(self):
        # List ports: -D.
        resources = "ports"
        cmd = port.ListPort(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_ports_pagination(self):
        resources = "ports"
        cmd = port.ListPort(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_ports_sort(self):
        # list ports:
        # --sort-key name --sort-key id --sort-key asc --sort-key desc
        resources = "ports"
        cmd = port.ListPort(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_ports_limit(self):
        # list ports: -P.
        resources = "ports"
        cmd = port.ListPort(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_list_ports_tags(self):
        # List ports: -- --tags a b.
        resources = "ports"
        cmd = port.ListPort(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, tags=['a', 'b'])

    def test_list_ports_detail_tags(self):
        # List ports: -D -- --tags a b.
        resources = "ports"
        cmd = port.ListPort(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, detail=True, tags=['a', 'b'])

    def test_list_ports_fields(self):
        # List ports: --fields a --fields b -- --fields c d.
        resources = "ports"
        cmd = port.ListPort(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  fields_1=['a', 'b'], fields_2=['c', 'd'])

    def test_list_ports_with_fixed_ips_in_csv(self):
        # List ports: -f csv.
        resources = "ports"
        cmd = port.ListPort(test_cli20.MyApp(sys.stdout), None)
        fixed_ips = [{"subnet_id": "30422057-d6df-4c90-8314-aefb5e326666",
                      "ip_address": "10.0.0.12"},
                     {"subnet_id": "30422057-d6df-4c90-8314-aefb5e326666",
                      "ip_address": "10.0.0.4"}]
        contents = [{'name': 'name1', 'fixed_ips': fixed_ips}]
        self._test_list_resources(resources, cmd, True,
                                  response_contents=contents,
                                  output_format='csv')

    def _test_list_router_port(self, resources, cmd,
                               myid, detail=False, tags=(),
                               fields_1=(), fields_2=()):
        reses = {resources: [{'id': 'myid1', },
                             {'id': 'myid2', }, ], }

        resstr = self.client.serialize(reses)
        # url method body
        query = ""
        args = detail and ['-D', ] or []

        if fields_1:
            for field in fields_1:
                args.append('--fields')
                args.append(field)
        args.append(myid)
        if tags:
            args.append('--')
            args.append("--tag")
        for tag in tags:
            args.append(tag)
        if (not tags) and fields_2:
            args.append('--')
        if fields_2:
            args.append("--fields")
            for field in fields_2:
                args.append(field)
        for field in itertools.chain(fields_1, fields_2):
            if query:
                query += "&fields=" + field
            else:
                query = "fields=" + field

        for tag in tags:
            if query:
                query += "&tag=" + tag
            else:
                query = "tag=" + tag
        if detail:
            query = query and query + '&verbose=True' or 'verbose=True'
        query = query and query + '&device_id=%s' or 'device_id=%s'
        path = getattr(self.client, resources + "_path")
        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=(test_cli20.MyResp(200),
                                                resstr)) as mock_request:
            cmd_parser = cmd.get_parser("list_" + resources)
            shell.run_command(cmd, cmd_parser, args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), 2)
        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(
                test_cli20.end_url(path, query % myid), self.client),
            'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))
        _str = self.fake_stdout.make_string()

        self.assertIn('myid1', _str)

    def test_list_router_ports(self):
        # List router ports: -D.
        resources = "ports"
        cmd = port.ListRouterPort(test_cli20.MyApp(sys.stdout), None)
        self._test_list_router_port(resources, cmd,
                                    self.test_id, True)

    def test_list_router_ports_tags(self):
        # List router ports: -- --tags a b.
        resources = "ports"
        cmd = port.ListRouterPort(test_cli20.MyApp(sys.stdout), None)
        self._test_list_router_port(resources, cmd,
                                    self.test_id, tags=['a', 'b'])

    def test_list_router_ports_detail_tags(self):
        # List router ports: -D -- --tags a b.
        resources = "ports"
        cmd = port.ListRouterPort(test_cli20.MyApp(sys.stdout), None)
        self._test_list_router_port(resources, cmd, self.test_id,
                                    detail=True, tags=['a', 'b'])

    def test_list_router_ports_fields(self):
        # List ports: --fields a --fields b -- --fields c d.
        resources = "ports"
        cmd = port.ListRouterPort(test_cli20.MyApp(sys.stdout), None)
        self._test_list_router_port(resources, cmd, self.test_id,
                                    fields_1=['a', 'b'],
                                    fields_2=['c', 'd'])

    def test_update_port(self):
        # Update port: myid --name myname --admin-state-up False --tags a b.
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--admin-state-up', 'False',
                                    '--description', 'garbage',
                                    '--tags', 'a', 'b'],
                                   {'name': 'myname',
                                    'admin_state_up': 'False',
                                    'description': 'garbage',
                                    'tags': ['a', 'b'], })

    def test_update_port_secgroup(self):
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = ['--security-group', 'sg1_id', myid]
        updatefields = {'security_groups': ['sg1_id']}
        self._test_update_resource(resource, cmd, myid, args, updatefields)

    def test_update_port_secgroups(self):
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = ['--security-group', 'sg1_id',
                '--security-group', 'sg2_id',
                myid]
        updatefields = {'security_groups': ['sg1_id', 'sg2_id']}
        self._test_update_resource(resource, cmd, myid, args, updatefields)

    def test_update_port_extra_dhcp_opts(self):
        # Update port: myid --extra_dhcp_opt.
        resource = 'port'
        myid = 'myid'
        args = [myid,
                '--extra-dhcp-opt',
                "opt_name=bootfile-name,opt_value=pxelinux.0",
                '--extra-dhcp-opt',
                "opt_name=tftp-server,opt_value=123.123.123.123",
                '--extra-dhcp-opt',
                "opt_name=server-ip-address,opt_value=123.123.123.45"
                ]
        updatedfields = {'extra_dhcp_opts': [{'opt_name': 'bootfile-name',
                                              'opt_value': 'pxelinux.0'},
                                             {'opt_name': 'tftp-server',
                                              'opt_value': '123.123.123.123'},
                                             {'opt_name': 'server-ip-address',
                                              'opt_value': '123.123.123.45'}]}
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, myid, args, updatedfields)

    def test_update_port_fixed_ip(self):
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        subnet_id = 'subnet_id'
        ip_addr = '123.123.123.123'
        args = [myid,
                '--fixed-ip',
                "subnet_id=%(subnet_id)s,ip_address=%(ip_addr)s" %
                {'subnet_id': subnet_id,
                 'ip_addr': ip_addr}]
        updated_fields = {"fixed_ips": [{'subnet_id': subnet_id,
                                         'ip_address': ip_addr}]}
        self._test_update_resource(resource, cmd, myid, args, updated_fields)

    def test_update_port_device_id_device_owner(self):
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = ['--device-id', 'dev_id', '--device-owner', 'fake', myid]
        updatefields = {'device_id': 'dev_id',
                        'device_owner': 'fake'}
        self._test_update_resource(resource, cmd, myid, args, updatefields)

    def test_update_port_extra_dhcp_opts_ip_version(self):
        # Update port: myid --extra_dhcp_opt.
        resource = 'port'
        myid = 'myid'
        args = [myid,
                '--extra-dhcp-opt',
                "opt_name=bootfile-name,opt_value=pxelinux.0,ip_version=4",
                '--extra-dhcp-opt',
                "opt_name=tftp-server,opt_value=2001:192:168::1,ip_version=6",
                '--extra-dhcp-opt',
                "opt_name=server-ip-address,opt_value=null,ip_version=4"
                ]
        updatedfields = {'extra_dhcp_opts': [{'opt_name': 'bootfile-name',
                                              'opt_value': 'pxelinux.0',
                                              'ip_version': '4'},
                                             {'opt_name': 'tftp-server',
                                              'opt_value': '2001:192:168::1',
                                              'ip_version': '6'},
                                             {'opt_name': 'server-ip-address',
                                              'opt_value': None,
                                              'ip_version': '4'}]}
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, myid, args, updatedfields)

    def test_update_port_with_qos_policy(self):
        # Update port: myid --qos-policy mypolicy.
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--qos-policy', 'mypolicy'],
                                   {'qos_policy_id': 'mypolicy', })

    def test_update_port_with_no_qos_policy(self):
        # Update port: myid --no-qos-policy.
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--no-qos-policy'],
                                   {'qos_policy_id': None, })

    def test_update_port_with_dns_name(self):
        # Update port: myid --dns-name my-port.
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--dns-name', 'my-port'],
                                   {'dns_name': 'my-port', })

    def test_update_port_with_no_dns_name(self):
        # Update port: myid --no-dns-name
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--no-dns-name'],
                                   {'dns_name': "", })

    def test_delete_extra_dhcp_opts_from_port(self):
        resource = 'port'
        myid = 'myid'
        args = [myid,
                '--extra-dhcp-opt',
                "opt_name=bootfile-name,opt_value=null",
                '--extra-dhcp-opt',
                "opt_name=tftp-server,opt_value=123.123.123.123",
                '--extra-dhcp-opt',
                "opt_name=server-ip-address,opt_value=123.123.123.45"
                ]
        # the client code will change the null to None and send to server,
        # where its interpreted as delete the DHCP option on the port.
        updatedfields = {'extra_dhcp_opts': [{'opt_name': 'bootfile-name',
                                             'opt_value': None},
                                             {'opt_name': 'tftp-server',
                                              'opt_value': '123.123.123.123'},
                                             {'opt_name': 'server-ip-address',
                                              'opt_value': '123.123.123.45'}]}
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, myid, args, updatedfields)

    def test_update_port_security_group_off(self):
        # Update port: --no-security-groups myid.
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['--no-security-groups', 'myid'],
                                   {'security_groups': []})

    def test_update_port_allowed_address_pair_ipaddr(self):
        # Update port(ip_address only):
        # --allowed-address-pairs ip_address=addr0
        # --allowed-address-pairs ip_address=addr1
        import sys
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        pairs = [{'ip_address': '123.123.123.123'},
                 {'ip_address': '123.123.123.45'}]
        args = [myid,
                '--allowed-address-pair',
                'ip_address=123.123.123.123',
                '--allowed-address-pair',
                'ip_address=123.123.123.45']
        updatefields = {'allowed_address_pairs': pairs}
        self._test_update_resource(resource, cmd, myid, args, updatefields)

    def test_update_port_allowed_address_pair(self):
        # Update port:
        # --allowed-address-pair ip_address=addr0,mac_address=mac0
        # --allowed-address-pair ip_address_addr1,mac_address=mac1
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        pairs = [{'ip_address': '123.123.123.123',
                  'mac_address': '10:00:00:00:00:00'},
                 {'ip_address': '123.123.123.45',
                  'mac_address': '10:00:00:00:00:01'}]
        args = [myid,
                '--allowed-address-pair',
                'ip_address=123.123.123.123,mac_address=10:00:00:00:00:00',
                '--allowed-address-pair',
                'ip_address=123.123.123.45,mac_address=10:00:00:00:00:01']
        updatefields = {'allowed_address_pairs': pairs}
        self._test_update_resource(resource, cmd, myid, args, updatefields)

    def test_update_port_allowed_address_pairs_off(self):
        # Update port: --no-allowed-address-pairs.
        resource = 'port'
        cmd = port.UpdatePort(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['--no-allowed-address-pairs', 'myid'],
                                   {'allowed_address_pairs': []})

    def test_show_port(self):
        # Show port: --fields id --fields name myid.
        resource = 'port'
        cmd = port.ShowPort(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_delete_port(self):
        # Delete port: myid.
        resource = 'port'
        cmd = port.DeletePort(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import contextlib
from io import StringIO
import itertools
import sys
from unittest import mock
import urllib.parse as urlparse

from oslo_serialization import jsonutils
from oslo_utils import encodeutils
from oslotest import base
import requests
import yaml

from neutronclient.common import constants
from neutronclient.common import exceptions
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV2_0
from neutronclient.neutron.v2_0 import network
from neutronclient import shell
from neutronclient.v2_0 import client

API_VERSION = "2.0"
TOKEN = 'testtoken'
ENDURL = 'localurl'
REQUEST_ID = 'test_request_id'


@contextlib.contextmanager
def capture_std_streams():
    fake_stdout, fake_stderr = StringIO(), StringIO()
    stdout, stderr = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = fake_stdout, fake_stderr
        yield fake_stdout, fake_stderr
    finally:
        sys.stdout, sys.stderr = stdout, stderr


class FakeStdout(object):

    def __init__(self):
        self.content = []

    def write(self, text):
        self.content.append(text)

    def make_string(self):
        result = ''
        for line in self.content:
            result += encodeutils.safe_decode(line, 'utf-8')
        return result


class MyRequest(requests.Request):
    def __init__(self, method=None):
        self.method = method


class MyResp(requests.Response):
    def __init__(self, status_code, headers=None, reason=None,
                 request=None, url=None):
        self.status_code = status_code
        self.headers = headers or {}
        self.reason = reason
        self.request = request or MyRequest()
        self.url = url


class MyApp(object):
    def __init__(self, _stdout):
        self.stdout = _stdout


def end_url(path, query=None):
    _url_str = ENDURL + "/v" + API_VERSION + path
    return query and _url_str + "?" + query or _url_str


class MyUrlComparator(object):
    def __init__(self, lhs, client):
        self.lhs = lhs
        self.client = client

    def __eq__(self, rhs):
        lhsp = urlparse.urlparse(self.lhs)
        rhsp = urlparse.urlparse(rhs)

        lhs_qs = urlparse.parse_qsl(lhsp.query)
        rhs_qs = urlparse.parse_qsl(rhsp.query)

        return (lhsp.scheme == rhsp.scheme and
                lhsp.netloc == rhsp.netloc and
                lhsp.path == rhsp.path and
                len(lhs_qs) == len(rhs_qs) and
                set(lhs_qs) == set(rhs_qs))

    def __str__(self):
        return self.lhs

    def __repr__(self):
        return str(self)


class MyComparator(object):
    def __init__(self, lhs, client):
        self.lhs = lhs
        self.client = client

    def _com_dict(self, lhs, rhs):
        if len(lhs) != len(rhs):
            return False
        for key, value in lhs.items():
            if key not in rhs:
                return False
            rhs_value = rhs[key]
            if not self._com(value, rhs_value):
                return False
        return True

    def _com_list(self, lhs, rhs):
        if len(lhs) != len(rhs):
            return False
        for lhs_value in lhs:
            if lhs_value not in rhs:
                return False
        return True

    def _com(self, lhs, rhs):
        if lhs is None:
            return rhs is None
        if isinstance(lhs, dict):
            if not isinstance(rhs, dict):
                return False
            return self._com_dict(lhs, rhs)
        if isinstance(lhs, list):
            if not isinstance(rhs, list):
                return False
            return self._com_list(lhs, rhs)
        if isinstance(lhs, tuple):
            if not isinstance(rhs, tuple):
                return False
            return self._com_list(lhs, rhs)
        return lhs == rhs

    def __eq__(self, rhs):
        if self.client:
            rhs = self.client.deserialize(rhs, 200)
        return self._com(self.lhs, rhs)

    def __repr__(self):
        if self.client:
            return self.client.serialize(self.lhs)
        return str(self.lhs)


class ContainsKeyValue(object):
    """Checks whether key/value pair(s) are included in a dict parameter.

    This class just checks whether specifid key/value pairs passed in
    __init__() are included in a dict parameter. The comparison does not
    fail even if other key/value pair(s) exists in a target dict.
    """

    def __init__(self, expected):
        self._expected = expected

    def __eq__(self, other):
        if not isinstance(other, dict):
            return False
        for key, value in self._expected.items():
            if key not in other:
                return False
            if other[key] != value:
                return False
        return True

    def __repr__(self):
        return ('<%s (expected: %s)>' %
                (self.__class__.__name__, self._expected))


class IsA(object):
    """Checks whether the parameter is of specific type."""

    def __init__(self, expected_type):
        self._expected_type = expected_type

    def __eq__(self, other):
        return isinstance(other, self._expected_type)

    def __repr__(self):
        return ('<%s (expected: %s)>' %
                (self.__class__.__name__, self._expected_type))


class CLITestV20Base(base.BaseTestCase):

    test_id = 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'
    id_field = 'id'

    non_admin_status_resources = []

    def _find_resourceid(self, client, resource, name_or_id,
                         cmd_resource=None, parent_id=None):
        return name_or_id

    def setUp(self, plurals=None):
        """Prepare the test environment."""
        super(CLITestV20Base, self).setUp()
        client.Client.EXTED_PLURALS.update(constants.PLURALS)
        if plurals is not None:
            client.Client.EXTED_PLURALS.update(plurals)
        self.metadata = {'plurals': client.Client.EXTED_PLURALS}
        self.endurl = ENDURL
        self.fake_stdout = FakeStdout()

        self.addCleanup(mock.patch.stopall)
        mock.patch('sys.stdout', new=self.fake_stdout).start()
        mock.patch('neutronclient.neutron.v2_0.find_resourceid_by_name_or_id',
                   new=self._find_resourceid).start()
        mock.patch('neutronclient.neutron.v2_0.find_resourceid_by_id',
                   new=self._find_resourceid).start()

        self.client = client.Client(token=TOKEN, endpoint_url=self.endurl)

    def register_non_admin_status_resource(self, resource_name):
        # TODO(amotoki):
        # It is recommended to define
        # "non_admin_status_resources in each test class rather than
        # using register_non_admin_status_resource method.

        # If we change self.non_admin_status_resources like this,
        # we need to ensure this should be an instance variable
        # to avoid changing the class variable.
        if (id(self.non_admin_status_resources) ==
                id(self.__class__.non_admin_status_resources)):
            self.non_admin_status_resources = (self.__class__.
                                               non_admin_status_resources[:])
        self.non_admin_status_resources.append(resource_name)

    def _test_create_resource(self, resource, cmd, name, myid, args,
                              position_names, position_values,
                              tenant_id=None, tags=None, admin_state_up=True,
                              extra_body=None, cmd_resource=None,
                              parent_id=None, no_api_call=False,
                              expected_exception=None,
                              **kwargs):
        if not cmd_resource:
            cmd_resource = resource
        if (resource in self.non_admin_status_resources):
            body = {resource: {}, }
        else:
            body = {resource: {'admin_state_up': admin_state_up, }, }
        if tenant_id:
            body[resource].update({'tenant_id': tenant_id})
        if tags:
            body[resource].update({'tags': tags})
        if extra_body:
            body[resource].update(extra_body)
        body[resource].update(kwargs)

        for i in range(len(position_names)):
            body[resource].update({position_names[i]: position_values[i]})
        ress = {resource:
                {self.id_field: myid}, }
        if name:
            ress[resource].update({'name': name})
        resstr = self.client.serialize(ress)
        # url method body
        resource_plural = self.client.get_resource_plural(cmd_resource)
        path = getattr(self.client, resource_plural + "_path")
        if parent_id:
            path = path % parent_id
        mock_body = MyComparator(body, self.client)

        cmd_parser = cmd.get_parser('create_' + resource)
        resp = (MyResp(200), resstr)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=resp) as mock_request:
            if expected_exception:
                self.assertRaises(expected_exception,
                                  shell.run_command, cmd, cmd_parser, args)
            else:
                shell.run_command(cmd, cmd_parser, args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), None)
        if not no_api_call:
            mock_request.assert_called_once_with(
                end_url(path), 'POST',
                body=mock_body,
                headers=ContainsKeyValue({'X-Auth-Token': TOKEN}))
        if not expected_exception:
            _str = self.fake_stdout.make_string()
            self.assertIn(myid, _str)
            if name:
                self.assertIn(name, _str)

    def _test_list_columns(self, cmd, resources,
                           resources_out, args=('-f', 'json'),
                           cmd_resources=None, parent_id=None):
        if not cmd_resources:
            cmd_resources = resources

        resstr = self.client.serialize(resources_out)

        path = getattr(self.client, cmd_resources + "_path")
        if parent_id:
            path = path % parent_id
        cmd_parser = cmd.get_parser("list_" + cmd_resources)
        resp = (MyResp(200), resstr)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=resp) as mock_request:
            shell.run_command(cmd, cmd_parser, args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), None)
        mock_request.assert_called_once_with(
            end_url(path), 'GET',
            body=None,
            headers=ContainsKeyValue({'X-Auth-Token': TOKEN}))

    def _test_list_resources(self, resources, cmd, detail=False, tags=(),
                             fields_1=(), fields_2=(), page_size=None,
                             sort_key=(), sort_dir=(), response_contents=None,
                             base_args=None, path=None, cmd_resources=None,
                             parent_id=None, output_format=None, query=""):
        if not cmd_resources:
            cmd_resources = resources
        if response_contents is None:
            contents = [{self.id_field: 'myid1', },
                        {self.id_field: 'myid2', }, ]
        else:
            contents = response_contents
        reses = {resources: contents}
        resstr = self.client.serialize(reses)
        # url method body
        args = base_args if base_args is not None else []
        if detail:
            args.append('-D')
        if fields_1:
            for field in fields_1:
                args.append('--fields')
                args.append(field)

        if tags:
            args.append('--')
            args.append("--tag")
        for tag in tags:
            args.append(tag)
            tag_query = urlparse.urlencode(
                {'tag': encodeutils.safe_encode(tag)})
            if query:
                query += "&" + tag_query
            else:
                query = tag_query
        if (not tags) and fields_2:
            args.append('--')
        if fields_2:
            args.append("--fields")
            for field in fields_2:
                args.append(field)
        if detail:
            query = query and query + '&verbose=True' or 'verbose=True'
        for field in itertools.chain(fields_1, fields_2):
            if query:
                query += "&fields=" + field
            else:
                query = "fields=" + field
        if page_size:
            args.append("--page-size")
            args.append(str(page_size))
            if query:
                query += "&limit=%s" % page_size
            else:
                query = "limit=%s" % page_size
        if sort_key:
            for key in sort_key:
                args.append('--sort-key')
                args.append(key)
                if query:
                    query += '&'
                query += 'sort_key=%s' % key
        if sort_dir:
            len_diff = len(sort_key) - len(sort_dir)
            if len_diff > 0:
                sort_dir = tuple(sort_dir) + ('asc',) * len_diff
            elif len_diff < 0:
                sort_dir = sort_dir[:len(sort_key)]
            for dir in sort_dir:
                args.append('--sort-dir')
                args.append(dir)
                if query:
                    query += '&'
                query += 'sort_dir=%s' % dir
        if path is None:
            path = getattr(self.client, cmd_resources + "_path")
            if parent_id:
                path = path % parent_id
        if output_format:
            args.append('-f')
            args.append(output_format)
        cmd_parser = cmd.get_parser("list_" + cmd_resources)
        resp = (MyResp(200), resstr)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=resp) as mock_request:
            shell.run_command(cmd, cmd_parser, args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), None)
        mock_request.assert_called_once_with(
            MyUrlComparator(end_url(path, query), self.client),
            'GET',
            body=None,
            headers=ContainsKeyValue({'X-Auth-Token': TOKEN}))
        _str = self.fake_stdout.make_string()
        if response_contents is None:
            self.assertIn('myid1', _str)
        return _str

    def _test_list_resources_with_pagination(self, resources, cmd,
                                             base_args=None,
                                             cmd_resources=None,
                                             parent_id=None, query=""):
        if not cmd_resources:
            cmd_resources = resources

        path = getattr(self.client, cmd_resources + "_path")
        if parent_id:
            path = path % parent_id
        fake_query = "marker=myid2&limit=2"
        reses1 = {resources: [{'id': 'myid1', },
                              {'id': 'myid2', }],
                  '%s_links' % resources: [{'href': end_url(path, fake_query),
                                            'rel': 'next'}]}
        reses2 = {resources: [{'id': 'myid3', },
                              {'id': 'myid4', }]}
        resstr1 = self.client.serialize(reses1)
        resstr2 = self.client.serialize(reses2)
        cmd_parser = cmd.get_parser("list_" + cmd_resources)
        args = base_args if base_args is not None else []
        mock_request_calls = [
            mock.call(
                end_url(path, query), 'GET',
                body=None,
                headers=ContainsKeyValue({'X-Auth-Token': TOKEN})),
            mock.call(
                MyUrlComparator(end_url(path, fake_query),
                                self.client), 'GET',
                body=None,
                headers=ContainsKeyValue({'X-Auth-Token': TOKEN}))]
        mock_request_resp = [(MyResp(200), resstr1), (MyResp(200), resstr2)]

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient,
                                  "request") as mock_request:
            mock_request.side_effect = mock_request_resp
            shell.run_command(cmd, cmd_parser, args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), None)
        self.assertEqual(2, mock_request.call_count)
        mock_request.assert_has_calls(mock_request_calls)

    def _test_update_resource(self, resource, cmd, myid, args, extrafields,
                              cmd_resource=None, parent_id=None):
        if not cmd_resource:
            cmd_resource = resource

        body = {resource: extrafields}
        path = getattr(self.client, cmd_resource + "_path")
        if parent_id:
            path = path % (parent_id, myid)
        else:
            path = path % myid
        mock_body = MyComparator(body, self.client)

        cmd_parser = cmd.get_parser("update_" + cmd_resource)
        resp = (MyResp(204), None)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=resp) as mock_request:
            shell.run_command(cmd, cmd_parser, args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), None)
        mock_request.assert_called_once_with(
            MyUrlComparator(end_url(path), self.client),
            'PUT',
            body=mock_body,
            headers=ContainsKeyValue({'X-Auth-Token': TOKEN}))
        _str = self.fake_stdout.make_string()
        self.assertIn(myid, _str)

    def _test_show_resource(self, resource, cmd, myid, args, fields=(),
                            cmd_resource=None, parent_id=None):
        if not cmd_resource:
            cmd_resource = resource

        query = "&".join(["fields=%s" % field for field in fields])
        expected_res = {resource:
                        {self.id_field: myid,
                         'name': 'myname', }, }
        resstr = self.client.serialize(expected_res)
        path = getattr(self.client, cmd_resource + "_path")
        if parent_id:
            path = path % (parent_id, myid)
        else:
            path = path % myid
        cmd_parser = cmd.get_parser("show_" + cmd_resource)
        resp = (MyResp(200), resstr)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=resp) as mock_request:
            shell.run_command(cmd, cmd_parser, args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), None)
        mock_request.assert_called_once_with(
            end_url(path, query), 'GET',
            body=None,
            headers=ContainsKeyValue({'X-Auth-Token': TOKEN}))
        _str = self.fake_stdout.make_string()
        self.assertIn(myid, _str)
        self.assertIn('myname', _str)

    def _test_set_path_and_delete(self, path, parent_id, myid,
                                  mock_request_calls, mock_request_returns,
                                  delete_fail=False):
        return_val = 404 if delete_fail else 204
        if parent_id:
            path = path % (parent_id, myid)
        else:
            path = path % (myid)
        mock_request_returns.append((MyResp(return_val), None))
        mock_request_calls.append(mock.call(
            end_url(path), 'DELETE',
            body=None,
            headers=ContainsKeyValue({'X-Auth-Token': TOKEN})))

    def _test_delete_resource(self, resource, cmd, myid, args,
                              cmd_resource=None, parent_id=None,
                              extra_id=None, delete_fail=False):
        mock_request_calls = []
        mock_request_returns = []
        if not cmd_resource:
            cmd_resource = resource
        path = getattr(self.client, cmd_resource + "_path")
        self._test_set_path_and_delete(path, parent_id, myid,
                                       mock_request_calls,
                                       mock_request_returns)
        # extra_id is used to test for bulk_delete
        if extra_id:
            self._test_set_path_and_delete(path, parent_id, extra_id,
                                           mock_request_calls,
                                           mock_request_returns,
                                           delete_fail)
        cmd_parser = cmd.get_parser("delete_" + cmd_resource)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient,
                                  "request") as mock_request:
            mock_request.side_effect = mock_request_returns
            shell.run_command(cmd, cmd_parser, args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), None)
        mock_request.assert_has_calls(mock_request_calls)
        _str = self.fake_stdout.make_string()
        self.assertIn(myid, _str)
        if extra_id:
            self.assertIn(extra_id, _str)

    def _test_update_resource_action(self, resource, cmd, myid, action, args,
                                     body, expected_code=200, retval=None,
                                     cmd_resource=None):
        if not cmd_resource:
            cmd_resource = resource
        path = getattr(self.client, cmd_resource + "_path")
        path_action = '%s/%s' % (myid, action)
        cmd_parser = cmd.get_parser("update_" + cmd_resource)
        resp = (MyResp(expected_code), retval)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=resp) as mock_request:
            shell.run_command(cmd, cmd_parser, args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), None)
        mock_request.assert_called_once_with(
            end_url(path % path_action), 'PUT',
            body=MyComparator(body, self.client),
            headers=ContainsKeyValue({'X-Auth-Token': TOKEN}))
        _str = self.fake_stdout.make_string()
        self.assertIn(myid, _str)

    def assert_mock_multiple_calls_with_same_arguments(
            self, mocked_method, expected_call, count):
        if count is None:
            self.assertLessEqual(1, mocked_method.call_count)
        else:
            self.assertEqual(count, mocked_method.call_count)
        mocked_method.assert_has_calls(
            [expected_call] * mocked_method.call_count)


class TestListCommand(neutronV2_0.ListCommand):
    resource = 'test_resource'
    filter_attrs = [
        'name',
        'admin_state_up',
        {'name': 'foo', 'help': 'non-boolean attribute foo'},
        {'name': 'bar', 'help': 'boolean attribute bar',
         'boolean': True},
        {'name': 'baz', 'help': 'integer attribute baz',
         'argparse_kwargs': {'choices': ['baz1', 'baz2']}},
    ]


class ListCommandTestCase(CLITestV20Base):

    def setUp(self):
        super(ListCommandTestCase, self).setUp()
        self.client.extend_list('test_resources', '/test_resources', None)
        setattr(self.client, 'test_resources_path', '/test_resources')

    def _test_list_resources_filter_params(self, base_args='', query=''):
        resources = 'test_resources'
        cmd = TestListCommand(MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  base_args=base_args.split(),
                                  query=query)

    def _test_list_resources_with_arg_error(self, base_args=''):
        resources = 'test_resources'
        cmd = TestListCommand(MyApp(sys.stdout), None)
        # argparse parse error leads to SystemExit
        self.assertRaises(SystemExit,
                          self._test_list_resources,
                          resources, cmd,
                          base_args=base_args.split())

    def test_list_resources_without_filter(self):
        self._test_list_resources_filter_params()

    def test_list_resources_use_default_filter(self):
        self._test_list_resources_filter_params(
            base_args='--name val1 --admin-state-up False',
            query='name=val1&admin_state_up=False')

    def test_list_resources_use_custom_filter(self):
        self._test_list_resources_filter_params(
            base_args='--foo FOO --bar True',
            query='foo=FOO&bar=True')

    def test_list_resources_boolean_check_default_filter(self):
        self._test_list_resources_filter_params(
            base_args='--admin-state-up True', query='admin_state_up=True')
        self._test_list_resources_filter_params(
            base_args='--admin-state-up False', query='admin_state_up=False')
        self._test_list_resources_with_arg_error(
            base_args='--admin-state-up non-true-false')

    def test_list_resources_boolean_check_custom_filter(self):
        self._test_list_resources_filter_params(
            base_args='--bar True', query='bar=True')
        self._test_list_resources_filter_params(
            base_args='--bar False', query='bar=False')
        self._test_list_resources_with_arg_error(
            base_args='--bar non-true-false')

    def test_list_resources_argparse_kwargs(self):
        self._test_list_resources_filter_params(
            base_args='--baz baz1', query='baz=baz1')
        self._test_list_resources_filter_params(
            base_args='--baz baz2', query='baz=baz2')
        self._test_list_resources_with_arg_error(
            base_args='--bar non-choice')


class ClientV2TestJson(CLITestV20Base):
    def test_do_request_unicode(self):
        unicode_text = u'\u7f51\u7edc'
        # url with unicode
        action = u'/test'
        expected_action = action
        # query string with unicode
        params = {'test': unicode_text}
        expect_query = urlparse.urlencode(utils.safe_encode_dict(params))
        # request body with unicode
        body = params
        expect_body = self.client.serialize(body)
        self.client.httpclient.auth_token = encodeutils.safe_encode(
            unicode_text)
        expected_auth_token = encodeutils.safe_encode(unicode_text)
        resp_headers = {'x-openstack-request-id': REQUEST_ID}
        resp = (MyResp(200, resp_headers), expect_body)

        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            result = self.client.do_request('PUT', action, body=body,
                                            params=params)

        mock_request.assert_called_once_with(
            end_url(expected_action, query=expect_query),
            'PUT', body=expect_body,
            headers=ContainsKeyValue({'X-Auth-Token': expected_auth_token}))
        # test response with unicode
        self.assertEqual(body, result)

    def test_do_request_error_without_response_body(self):
        params = {'test': 'value'}
        expect_query = urlparse.urlencode(params)
        self.client.httpclient.auth_token = 'token'
        resp_headers = {'x-openstack-request-id': REQUEST_ID}
        resp = (MyResp(400, headers=resp_headers, reason='An error'), '')

        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            error = self.assertRaises(exceptions.NeutronClientException,
                                      self.client.do_request, 'PUT', '/test',
                                      body='', params=params)

        mock_request.assert_called_once_with(
            MyUrlComparator(end_url('/test', query=expect_query), self.client),
            'PUT', body='',
            headers=ContainsKeyValue({'X-Auth-Token': 'token'}))
        expected_error = "An error\nNeutron server returns " \
                         "request_ids: %s" % [REQUEST_ID]
        self.assertEqual(expected_error, str(error))

    def test_do_request_with_long_uri_exception(self):
        long_string = 'x' * 8200                  # 8200 > MAX_URI_LEN:8192
        params = {'id': long_string}
        exception = self.assertRaises(exceptions.RequestURITooLong,
                                      self.client.do_request,
                                      'GET', '/test', body='', params=params)
        self.assertNotEqual(0, exception.excess)

    def test_do_request_request_ids(self):
        params = {'test': 'value'}
        expect_query = urlparse.urlencode(params)
        self.client.httpclient.auth_token = 'token'
        body = params
        expect_body = self.client.serialize(body)
        resp_headers = {'x-openstack-request-id': REQUEST_ID}
        resp = (MyResp(200, resp_headers), expect_body)

        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            result = self.client.do_request('PUT', '/test', body=body,
                                            params=params)

        mock_request.assert_called_once_with(
            MyUrlComparator(end_url('/test', query=expect_query), self.client),
            'PUT', body=expect_body,
            headers=ContainsKeyValue({'X-Auth-Token': 'token'}))

        self.assertEqual(body, result)
        self.assertEqual([REQUEST_ID], result.request_ids)

    def test_list_request_ids_with_retrieve_all_true(self):
        path = '/test'
        resources = 'tests'
        fake_query = "marker=myid2&limit=2"
        reses1 = {resources: [{'id': 'myid1', },
                              {'id': 'myid2', }],
                  '%s_links' % resources: [{'href': end_url(path, fake_query),
                                            'rel': 'next'}]}
        reses2 = {resources: [{'id': 'myid3', },
                              {'id': 'myid4', }]}
        resstr1 = self.client.serialize(reses1)
        resstr2 = self.client.serialize(reses2)
        resp_headers = {'x-openstack-request-id': REQUEST_ID}
        resp = [(MyResp(200, resp_headers), resstr1),
                (MyResp(200, resp_headers), resstr2)]
        with mock.patch.object(self.client.httpclient, "request",
                               side_effect=resp) as mock_request:
            result = self.client.list(resources, path)

        self.assertEqual(2, mock_request.call_count)
        mock_request.assert_has_calls([
            mock.call(
                end_url(path, ""), 'GET',
                body=None,
                headers=ContainsKeyValue({'X-Auth-Token': TOKEN})),
            mock.call(
                MyUrlComparator(end_url(path, fake_query),
                                self.client), 'GET',
                body=None,
                headers=ContainsKeyValue({'X-Auth-Token': TOKEN}))])

        self.assertEqual([REQUEST_ID, REQUEST_ID], result.request_ids)

    def test_list_request_ids_with_retrieve_all_false(self):
        path = '/test'
        resources = 'tests'
        fake_query = "marker=myid2&limit=2"
        reses1 = {resources: [{'id': 'myid1', },
                              {'id': 'myid2', }],
                  '%s_links' % resources: [{'href': end_url(path, fake_query),
                                            'rel': 'next'}]}
        reses2 = {resources: [{'id': 'myid3', },
                              {'id': 'myid4', }]}
        resstr1 = self.client.serialize(reses1)
        resstr2 = self.client.serialize(reses2)
        resp_headers = {'x-openstack-request-id': REQUEST_ID}
        resp = [(MyResp(200, resp_headers), resstr1),
                (MyResp(200, resp_headers), resstr2)]
        with mock.patch.object(self.client.httpclient, "request",
                               side_effect=resp) as mock_request:
            result = self.client.list(resources, path, retrieve_all=False)
            next(result)
            self.assertEqual([REQUEST_ID], result.request_ids)
            next(result)
            self.assertEqual([REQUEST_ID, REQUEST_ID], result.request_ids)

        self.assertEqual(2, mock_request.call_count)
        mock_request.assert_has_calls([
            mock.call(
                end_url(path, ""), 'GET',
                body=None,
                headers=ContainsKeyValue({'X-Auth-Token': TOKEN})),
            mock.call(
                MyUrlComparator(end_url(path, fake_query), self.client), 'GET',
                body=None,
                headers=ContainsKeyValue({'X-Auth-Token': TOKEN}))])

    def test_deserialize_without_data(self):
        data = u''
        result = self.client.deserialize(data, 200)
        self.assertEqual(data, result)

    def test_update_resource(self):
        params = {'test': 'value'}
        expect_query = urlparse.urlencode(params)
        self.client.httpclient.auth_token = 'token'
        body = params
        expect_body = self.client.serialize(body)
        resp_headers = {'x-openstack-request-id': REQUEST_ID}
        resp = (MyResp(200, resp_headers), expect_body)

        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            result = self.client._update_resource('/test', body=body,
                                                  params=params)

        mock_request.assert_called_once_with(
            MyUrlComparator(end_url('/test', query=expect_query), self.client),
            'PUT', body=expect_body,
            headers=ContainsKeyValue({'X-Auth-Token': 'token'}))
        self.assertNotIn('If-Match', mock_request.call_args[1]['headers'])

        self.assertEqual(body, result)
        self.assertEqual([REQUEST_ID], result.request_ids)

    def test_update_resource_with_revision_number(self):
        params = {'test': 'value'}
        expect_query = urlparse.urlencode(params)
        self.client.httpclient.auth_token = 'token'
        body = params
        expect_body = self.client.serialize(body)
        resp_headers = {'x-openstack-request-id': REQUEST_ID}
        resp = (MyResp(200, resp_headers), expect_body)

        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            result = self.client._update_resource('/test', body=body,
                                                  params=params,
                                                  revision_number=1)

        mock_request.assert_called_once_with(
            MyUrlComparator(end_url('/test', query=expect_query), self.client),
            'PUT', body=expect_body,
            headers=ContainsKeyValue(
                {'X-Auth-Token': 'token', 'If-Match': 'revision_number=1'}))

        self.assertEqual(body, result)
        self.assertEqual([REQUEST_ID], result.request_ids)


class CLITestV20ExceptionHandler(CLITestV20Base):

    def _test_exception_handler_v20(
            self, expected_exception, status_code, expected_msg,
            error_type=None, error_msg=None, error_detail=None,
            request_id=None, error_content=None):

        resp = MyResp(status_code, {'x-openstack-request-id': request_id})
        if request_id is not None:
            expected_msg += "\nNeutron server returns " \
                            "request_ids: %s" % [request_id]
        if error_content is None:
            error_content = {'NeutronError': {'type': error_type,
                                              'message': error_msg,
                                              'detail': error_detail}}
        expected_content = self.client._convert_into_with_meta(error_content,
                                                               resp)

        e = self.assertRaises(expected_exception,
                              client.exception_handler_v20,
                              status_code, expected_content)
        self.assertEqual(status_code, e.status_code)
        self.assertEqual(expected_exception.__name__,
                         e.__class__.__name__)

        if expected_msg is None:
            if error_detail:
                expected_msg = '\n'.join([error_msg, error_detail])
            else:
                expected_msg = error_msg
        self.assertEqual(expected_msg, e.message)

    def test_exception_handler_v20_ip_address_in_use(self):
        err_msg = ('Unable to complete operation for network '
                   'fake-network-uuid. The IP address fake-ip is in use.')
        self._test_exception_handler_v20(
            exceptions.IpAddressInUseClient, 409, err_msg,
            'IpAddressInUse', err_msg, '', REQUEST_ID)

    def test_exception_handler_v20_neutron_known_error(self):
        known_error_map = [
            ('NetworkNotFound', exceptions.NetworkNotFoundClient, 404),
            ('PortNotFound', exceptions.PortNotFoundClient, 404),
            ('NetworkInUse', exceptions.NetworkInUseClient, 409),
            ('PortInUse', exceptions.PortInUseClient, 409),
            ('StateInvalid', exceptions.StateInvalidClient, 400),
            ('IpAddressInUse', exceptions.IpAddressInUseClient, 409),
            ('IpAddressGenerationFailure',
             exceptions.IpAddressGenerationFailureClient, 409),
            ('MacAddressInUse', exceptions.MacAddressInUseClient, 409),
            ('ExternalIpAddressExhausted',
             exceptions.ExternalIpAddressExhaustedClient, 400),
            ('OverQuota', exceptions.OverQuotaClient, 409),
            ('InvalidIpForNetwork', exceptions.InvalidIpForNetworkClient, 400),
            ('InvalidIpForSubnet', exceptions.InvalidIpForSubnetClient, 400),
            ('IpAddressAlreadyAllocated',
             exceptions.IpAddressAlreadyAllocatedClient, 400),
        ]

        error_msg = 'dummy exception message'
        error_detail = 'sample detail'
        for server_exc, client_exc, status_code in known_error_map:
            self._test_exception_handler_v20(
                client_exc, status_code,
                error_msg + '\n' + error_detail,
                server_exc, error_msg, error_detail, REQUEST_ID)

    def test_exception_handler_v20_neutron_known_error_without_detail(self):
        error_msg = 'Network not found'
        error_detail = ''
        self._test_exception_handler_v20(
            exceptions.NetworkNotFoundClient, 404,
            error_msg,
            'NetworkNotFound', error_msg, error_detail, REQUEST_ID)

    def test_exception_handler_v20_unknown_error_to_per_code_exception(self):
        for status_code, client_exc in exceptions.HTTP_EXCEPTION_MAP.items():
            error_msg = 'Unknown error'
            error_detail = 'This is detail'
            self._test_exception_handler_v20(
                client_exc, status_code,
                error_msg + '\n' + error_detail,
                'UnknownError', error_msg, error_detail, [REQUEST_ID])

    def test_exception_handler_v20_neutron_unknown_status_code(self):
        error_msg = 'Unknown error'
        error_detail = 'This is detail'
        self._test_exception_handler_v20(
            exceptions.NeutronClientException, 501,
            error_msg + '\n' + error_detail,
            'UnknownError', error_msg, error_detail, REQUEST_ID)

    def test_exception_handler_v20_bad_neutron_error(self):
        for status_code, client_exc in exceptions.HTTP_EXCEPTION_MAP.items():
            error_content = {'NeutronError': {'unknown_key': 'UNKNOWN'}}
            self._test_exception_handler_v20(
                client_exc, status_code,
                expected_msg="{'unknown_key': 'UNKNOWN'}",
                error_content=error_content,
                request_id=REQUEST_ID)

    def test_exception_handler_v20_error_dict_contains_message(self):
        error_content = {'message': 'This is an error message'}
        for status_code, client_exc in exceptions.HTTP_EXCEPTION_MAP.items():
            self._test_exception_handler_v20(
                client_exc, status_code,
                expected_msg='This is an error message',
                error_content=error_content,
                request_id=REQUEST_ID)

    def test_exception_handler_v20_error_dict_not_contain_message(self):
        # 599 is not contained in HTTP_EXCEPTION_MAP.
        error_content = {'error': 'This is an error message'}
        expected_msg = '%s-%s' % (599, error_content)
        self._test_exception_handler_v20(
            exceptions.NeutronClientException, 599,
            expected_msg=expected_msg,
            request_id=None,
            error_content=error_content)

    def test_exception_handler_v20_default_fallback(self):
        # 599 is not contained in HTTP_EXCEPTION_MAP.
        error_content = 'This is an error message'
        expected_msg = '%s-%s' % (599, error_content)
        self._test_exception_handler_v20(
            exceptions.NeutronClientException, 599,
            expected_msg=expected_msg,
            request_id=None,
            error_content=error_content)

    def test_exception_status(self):
        e = exceptions.BadRequest()
        self.assertEqual(400, e.status_code)

        e = exceptions.BadRequest(status_code=499)
        self.assertEqual(499, e.status_code)

        # SslCertificateValidationError has no explicit status_code,
        # but should have a 'safe' defined fallback.
        e = exceptions.SslCertificateValidationError()
        self.assertIsNotNone(e.status_code)

        e = exceptions.SslCertificateValidationError(status_code=599)
        self.assertEqual(599, e.status_code)

    def test_connection_failed(self):
        self.client.httpclient.auth_token = 'token'
        excp = requests.exceptions.ConnectionError('Connection refused')

        with mock.patch.object(self.client.httpclient, "request",
                               side_effect=excp) as mock_request:
            error = self.assertRaises(exceptions.ConnectionFailed,
                                      self.client.get, '/test')

        mock_request.assert_called_once_with(
            end_url('/test'), 'GET',
            body=None,
            headers=ContainsKeyValue({'X-Auth-Token': 'token'}))
        # NB: ConnectionFailed has no explicit status_code, so this
        # tests that there is a fallback defined.
        self.assertIsNotNone(error.status_code)


class DictWithMetaTest(base.BaseTestCase):

    def test_dict_with_meta(self):
        body = {'test': 'value'}
        resp = MyResp(200, {'x-openstack-request-id': REQUEST_ID})
        obj = client._DictWithMeta(body, resp)
        self.assertEqual(body, obj)

        # Check request_ids attribute is added to obj
        self.assertTrue(hasattr(obj, 'request_ids'))
        self.assertEqual([REQUEST_ID], obj.request_ids)


class TupleWithMetaTest(base.BaseTestCase):

    def test_tuple_with_meta(self):
        body = ('test', 'value')
        resp = MyResp(200, {'x-openstack-request-id': REQUEST_ID})
        obj = client._TupleWithMeta(body, resp)
        self.assertEqual(body, obj)

        # Check request_ids attribute is added to obj
        self.assertTrue(hasattr(obj, 'request_ids'))
        self.assertEqual([REQUEST_ID], obj.request_ids)


class StrWithMetaTest(base.BaseTestCase):

    def test_str_with_meta(self):
        body = "test_string"
        resp = MyResp(200, {'x-openstack-request-id': REQUEST_ID})
        obj = client._StrWithMeta(body, resp)
        self.assertEqual(body, obj)

        # Check request_ids attribute is added to obj
        self.assertTrue(hasattr(obj, 'request_ids'))
        self.assertEqual([REQUEST_ID], obj.request_ids)


class GeneratorWithMetaTest(base.BaseTestCase):

    body = {'test': 'value'}
    resp = MyResp(200, {'x-openstack-request-id': REQUEST_ID})

    def _pagination(self, collection, path, **params):
        obj = client._DictWithMeta(self.body, self.resp)
        yield obj

    def test_generator(self):
        obj = client._GeneratorWithMeta(self._pagination, 'test_collection',
                                        'test_path', test_args='test_args')
        self.assertEqual(self.body, next(obj))

        self.assertTrue(hasattr(obj, 'request_ids'))
        self.assertEqual([REQUEST_ID], obj.request_ids)


class CLITestV20OutputFormatter(CLITestV20Base):

    def _test_create_resource_with_formatter(self, fmt):
        resource = 'network'
        cmd = network.CreateNetwork(MyApp(sys.stdout), None)
        args = ['-f', fmt, 'myname']
        position_names = ['name']
        position_values = ['myname']
        self._test_create_resource(resource, cmd, 'myname', 'myid', args,
                                   position_names, position_values)

    def test_create_resource_table(self):
        self._test_create_resource_with_formatter('table')
        print(self.fake_stdout.content)
        # table data is contains in the third element.
        data = self.fake_stdout.content[2].split('\n')
        self.assertTrue(any(' id ' in d for d in data))
        self.assertTrue(any(' name ' in d for d in data))

    def test_create_resource_json(self):
        self._test_create_resource_with_formatter('json')
        data = jsonutils.loads(self.fake_stdout.make_string())
        self.assertEqual('myname', data['name'])
        self.assertEqual('myid', data['id'])

    def test_create_resource_yaml(self):
        self._test_create_resource_with_formatter('yaml')
        data = yaml.load(self.fake_stdout.make_string())
        self.assertEqual('myname', data['name'])
        self.assertEqual('myid', data['id'])

    def _test_show_resource_with_formatter(self, fmt):
        resource = 'network'
        cmd = network.ShowNetwork(MyApp(sys.stdout), None)
        args = ['-f', fmt, '-F', 'id', '-F', 'name', 'myid']
        self._test_show_resource(resource, cmd, 'myid',
                                 args, ['id', 'name'])

    def test_show_resource_table(self):
        self._test_show_resource_with_formatter('table')
        data = self.fake_stdout.content[0].split('\n')
        self.assertTrue(any(' id ' in d for d in data))
        self.assertTrue(any(' name ' in d for d in data))

    def test_show_resource_json(self):
        self._test_show_resource_with_formatter('json')
        data = jsonutils.loads(''.join(self.fake_stdout.content))
        self.assertEqual('myname', data['name'])
        self.assertEqual('myid', data['id'])

    def test_show_resource_yaml(self):
        self._test_show_resource_with_formatter('yaml')
        data = yaml.load(''.join(self.fake_stdout.content))
        self.assertEqual('myname', data['name'])
        self.assertEqual('myid', data['id'])

    @mock.patch.object(network.ListNetwork, "extend_list")
    def _test_list_resources_with_formatter(self, fmt, mock_extend_list):
        resources = 'networks'
        cmd = network.ListNetwork(MyApp(sys.stdout), None)
        # ListNetwork has its own extend_list, so we need to stub out it
        # to avoid an extra API call.
        self._test_list_resources(resources, cmd, output_format=fmt)
        mock_extend_list.assert_called_once_with(IsA(list), mock.ANY)

    def test_list_resources_table(self):
        self._test_list_resources_with_formatter('table')
        data = self.fake_stdout.content[0].split('\n')
        self.assertTrue(any(' id ' in d for d in data))
        self.assertTrue(any(' myid1 ' in d for d in data))
        self.assertTrue(any(' myid2 ' in d for d in data))

    def test_list_resources_json(self):
        self._test_list_resources_with_formatter('json')
        data = jsonutils.loads(''.join(self.fake_stdout.content))
        self.assertEqual(['myid1', 'myid2'], [d['id'] for d in data])

    def test_list_resources_yaml(self):
        self._test_list_resources_with_formatter('yaml')
        data = yaml.load(''.join(self.fake_stdout.content))
        self.assertEqual(['myid1', 'myid2'], [d['id'] for d in data])



# Copyright 2013 Mirantis Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronV20


class ListHealthMonitor(neutronV20.ListCommand):
    """List health monitors that belong to a given tenant."""

    resource = 'health_monitor'
    list_columns = ['id', 'type', 'admin_state_up']
    pagination_support = True
    sorting_support = True


class ShowHealthMonitor(neutronV20.ShowCommand):
    """Show information of a given health monitor."""

    resource = 'health_monitor'
    allow_names = False


class CreateHealthMonitor(neutronV20.CreateCommand):
    """Create a health monitor."""

    resource = 'health_monitor'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state', action='store_false',
            help=_('Set admin state up to false.'))
        parser.add_argument(
            '--expected-codes',
            help=_('The list of HTTP status codes expected in '
                   'response from the member to declare it healthy. This '
                   'attribute can contain one value, '
                   'or a list of values separated by comma, '
                   'or a range of values (e.g. "200-299"). If this attribute '
                   'is not specified, it defaults to "200".'))
        parser.add_argument(
            '--http-method',
            help=_('The HTTP method used for requests by the monitor of type '
                   'HTTP.'))
        parser.add_argument(
            '--url-path',
            help=_('The HTTP path used in the HTTP request used by the monitor'
                   ' to test a member health. This must be a string '
                   'beginning with a / (forward slash).'))
        parser.add_argument(
            '--delay',
            required=True,
            help=_('The time in seconds between sending probes to members.'))
        parser.add_argument(
            '--max-retries',
            required=True,
            help=_('Number of permissible connection failures before changing '
                   'the member status to INACTIVE. [1..10]'))
        parser.add_argument(
            '--timeout',
            required=True,
            help=_('Maximum number of seconds for a monitor to wait for a '
                   'connection to be established before it times out. The '
                   'value must be less than the delay value.'))
        parser.add_argument(
            '--type',
            required=True, choices=['PING', 'TCP', 'HTTP', 'HTTPS'],
            help=_('One of the predefined health monitor types.'))

    def args2body(self, parsed_args):
        body = {'admin_state_up': parsed_args.admin_state,
                'delay': parsed_args.delay,
                'max_retries': parsed_args.max_retries,
                'timeout': parsed_args.timeout,
                'type': parsed_args.type}
        neutronV20.update_dict(parsed_args, body,
                               ['expected_codes', 'http_method', 'url_path',
                                'tenant_id'])
        return {self.resource: body}


class UpdateHealthMonitor(neutronV20.UpdateCommand):
    """Update a given health monitor."""

    resource = 'health_monitor'
    allow_names = False


class DeleteHealthMonitor(neutronV20.DeleteCommand):
    """Delete a given health monitor."""

    resource = 'health_monitor'
    allow_names = False


class AssociateHealthMonitor(neutronV20.NeutronCommand):
    """Create a mapping between a health monitor and a pool."""

    resource = 'health_monitor'

    def get_parser(self, prog_name):
        parser = super(AssociateHealthMonitor, self).get_parser(prog_name)
        parser.add_argument(
            'health_monitor_id', metavar='HEALTH_MONITOR_ID',
            help=_('Health monitor to associate.'))
        parser.add_argument(
            'pool_id', metavar='POOL',
            help=_('ID of the pool to be associated with the health monitor.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        body = {'health_monitor': {'id': parsed_args.health_monitor_id}}
        pool_id = neutronV20.find_resourceid_by_name_or_id(
            neutron_client, 'pool', parsed_args.pool_id)
        neutron_client.associate_health_monitor(pool_id, body)
        print((_('Associated health monitor '
                 '%s') % parsed_args.health_monitor_id),
              file=self.app.stdout)


class DisassociateHealthMonitor(neutronV20.NeutronCommand):
    """Remove a mapping from a health monitor to a pool."""

    resource = 'health_monitor'

    def get_parser(self, prog_name):
        parser = super(DisassociateHealthMonitor, self).get_parser(prog_name)
        parser.add_argument(
            'health_monitor_id', metavar='HEALTH_MONITOR_ID',
            help=_('Health monitor to disassociate.'))
        parser.add_argument(
            'pool_id', metavar='POOL',
            help=_('ID of the pool to be disassociated with the health '
                   'monitor.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        pool_id = neutronV20.find_resourceid_by_name_or_id(
            neutron_client, 'pool', parsed_args.pool_id)
        neutron_client.disassociate_health_monitor(pool_id,
                                                   parsed_args
                                                   .health_monitor_id)
        print((_('Disassociated health monitor '
                 '%s') % parsed_args.health_monitor_id),
              file=self.app.stdout)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock
import uuid


from neutronclient.tests.unit.osc.v2 import fakes


class TestNeutronDynamicRoutingOSCV2(fakes.TestNeutronClientOSCV2):
    def setUp(self):
        super(TestNeutronDynamicRoutingOSCV2, self).setUp()
        self.neutronclient.find_resource = mock.Mock(
            side_effect=lambda resource, name_or_id, project_id=None,
            cmd_resource=None, parent_id=None, fields=None:
            {'id': name_or_id})


class FakeBgpSpeaker(object):
    """Fake one or more bgp speakers."""

    @staticmethod
    def create_one_bgp_speaker(attrs=None):
        attrs = attrs or {}
        # Set default attributes.
        bgp_speaker_attrs = {
            'peers': [],
            'local_as': 200,
            'advertise_tenant_networks': True,
            'networks': [],
            'ip_version': 4,
            'advertise_floating_ip_host_routes': True,
            'id': uuid.uuid4().hex,
            'name': 'bgp-speaker-' + uuid.uuid4().hex,
            'tenant_id': uuid.uuid4().hex,
        }

        # Overwrite default attributes.
        bgp_speaker_attrs.update(attrs)

        return copy.deepcopy(bgp_speaker_attrs)

    @staticmethod
    def create_bgp_speakers(attrs=None, count=1):
        """Create multiple fake bgp speakers.

        """
        bgp_speakers = []
        for i in range(count):
            bgp_speaker = FakeBgpSpeaker.create_one_bgp_speaker(attrs)
            bgp_speakers.append(bgp_speaker)

        return {'bgp_speakers': bgp_speakers}


class FakeBgpPeer(object):
    """Fake one or more bgp peers."""

    @staticmethod
    def create_one_bgp_peer(attrs=None):
        attrs = attrs or {}
        # Set default attributes.
        bgp_peer_attrs = {
            'auth_type': None,
            'peer_ip': '1.1.1.1',
            'remote_as': 100,
            'id': uuid.uuid4().hex,
            'name': 'bgp-peer-' + uuid.uuid4().hex,
            'tenant_id': uuid.uuid4().hex,
        }

        # Overwrite default attributes.
        bgp_peer_attrs.update(attrs)

        return copy.deepcopy(bgp_peer_attrs)

    @staticmethod
    def create_bgp_peers(attrs=None, count=1):
        """Create one or multiple fake bgp peers."""
        bgp_peers = []
        for i in range(count):
            bgp_peer = FakeBgpPeer.create_one_bgp_peer(attrs)
            bgp_peers.append(bgp_peer)

        return {'bgp_peers': bgp_peers}


class FakeDRAgent(object):
    """Fake one or more dynamic routing agents."""

    @staticmethod
    def create_one_dragent(attrs=None):
        attrs = attrs or {}
        # Set default attributes.
        dragent_attrs = {
            'binary': 'neutron-bgp-dragent',
            'admin_state_up': True,
            'alive': True,
            'topic': 'bgp_dragent',
            'host': 'network-' + uuid.uuid4().hex,
            'name': 'bgp-dragent-' + uuid.uuid4().hex,
            'agent_type': 'BGP dynamic routing agent',
            'id': uuid.uuid4().hex,
        }

        # Overwrite default attributes.
        dragent_attrs.update(attrs)

        return copy.deepcopy(dragent_attrs)

    @staticmethod
    def create_dragents(attrs=None, count=1):
        """Create one or multiple fake dynamic routing agents."""
        agents = []
        for i in range(count):
            agent = FakeDRAgent.create_one_dragent(attrs)
            agents.append(agent)

        return {'agents': agents}



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys
from unittest import mock

from oslo_utils import encodeutils
import testtools

from neutronclient._i18n import _
from neutronclient.common import exceptions


class TestExceptions(testtools.TestCase):

    def test_exception_print_with_unicode(self):
        class TestException(exceptions.NeutronException):
            message = _('Exception with %(reason)s')

        multibyte_unicode_string = u'\uff21\uff22\uff23'
        e = TestException(reason=multibyte_unicode_string)

        with mock.patch.object(sys, 'stdout') as mock_stdout:
            print(e)

        exc_str = 'Exception with %s' % multibyte_unicode_string
        mock_stdout.assert_has_calls([mock.call.write(exc_str)])

    def test_exception_message_with_encoded_unicode(self):
        class TestException(exceptions.NeutronException):
            message = _('Exception with %(reason)s')

        multibyte_string = u'\uff21\uff22\uff23'
        multibyte_binary = encodeutils.safe_encode(multibyte_string)
        e = TestException(reason=multibyte_binary)
        self.assertEqual('Exception with %s' % multibyte_string,
                         str(e))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import argparse
from unittest import mock

from cliff import columns as cliff_columns
from osc_lib.tests import utils


class TestNeutronClientOSCV2(utils.TestCommand):

    def setUp(self):
        super(TestNeutronClientOSCV2, self).setUp()
        self.namespace = argparse.Namespace()
        self.app.client_manager.session = mock.Mock()
        self.app.client_manager.neutronclient = mock.Mock()
        self.neutronclient = self.app.client_manager.neutronclient
        self.addCleanup(mock.patch.stopall)

    # TODO(amotoki): Move this to osc_lib
    def assertListItemEqual(self, expected, actual):
        self.assertEqual(len(expected), len(actual))
        for item_expected, item_actual in zip(expected, actual):
            self.assertItemEqual(item_expected, item_actual)

    # TODO(amotoki): Move this to osc_lib
    def assertItemEqual(self, expected, actual):
        self.assertEqual(len(expected), len(actual))
        for col_expected, col_actual in zip(expected, actual):
            if isinstance(col_expected, cliff_columns.FormattableColumn):
                self.assertIsInstance(col_actual, col_expected.__class__)
                self.assertEqual(col_expected.human_readable(),
                                 col_actual.human_readable())
            else:
                self.assertEqual(col_expected, col_actual)



# Copyright 2015 Huawei Technologies India Pvt Ltd.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.qos import policy as policy
from neutronclient.tests.unit import test_cli20


class CLITestV20QoSPolicyJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['policy']

    def setUp(self):
        super(CLITestV20QoSPolicyJSON, self).setUp()
        self.res = 'policy'
        self.cmd_res = 'qos_policy'
        self.ress = "policies"
        self.cmd_ress = 'qos_policies'

    def test_create_policy_with_only_keyattributes(self):
        # Create qos policy abc.
        cmd = policy.CreateQoSPolicy(test_cli20.MyApp(sys.stdout),
                                     None)
        myid = 'myid'
        name = 'abc'
        args = [name]
        position_names = ['name']
        position_values = [name]
        self._test_create_resource(self.res, cmd, name, myid, args,
                                   position_names, position_values,
                                   cmd_resource=self.cmd_res)

    def test_create_policy_with_description(self):
        # Create qos policy xyz --description abc.
        cmd = policy.CreateQoSPolicy(test_cli20.MyApp(sys.stdout),
                                     None)
        myid = 'myid'
        name = 'abc'
        description = 'policy_abc'
        args = [name, '--description', description]
        position_names = ['name', 'description']
        position_values = [name, description]
        self._test_create_resource(self.res, cmd, name, myid, args,
                                   position_names, position_values,
                                   cmd_resource=self.cmd_res)

    def test_create_policy_with_shared(self):
        # Create qos policy abc shared across tenants
        cmd = policy.CreateQoSPolicy(test_cli20.MyApp(sys.stdout),
                                     None)
        myid = 'myid'
        name = 'abc'
        description = 'policy_abc'
        args = [name, '--description', description, '--shared']
        position_names = ['name', 'description', 'shared']
        position_values = [name, description, True]
        self._test_create_resource(self.res, cmd, name, myid, args,
                                   position_names, position_values,
                                   cmd_resource=self.cmd_res)

    def test_create_policy_with_unicode(self):
        # Create qos policy u'\u7f51\u7edc'.
        cmd = policy.CreateQoSPolicy(test_cli20.MyApp(sys.stdout),
                                     None)
        myid = 'myid'
        name = u'\u7f51\u7edc'
        description = u'\u7f51\u7edc'
        args = [name, '--description', description]
        position_names = ['name', 'description']
        position_values = [name, description]
        self._test_create_resource(self.res, cmd, name, myid, args,
                                   position_names, position_values,
                                   cmd_resource=self.cmd_res)

    def test_update_policy(self):
        # policy-update myid --name newname.
        cmd = policy.UpdateQoSPolicy(test_cli20.MyApp(sys.stdout),
                                     None)
        self._test_update_resource(self.res, cmd, 'myid',
                                   ['myid', '--name', 'newname'],
                                   {'name': 'newname', },
                                   cmd_resource=self.cmd_res)

    def test_update_policy_description(self):
        # policy-update myid --name newname --description newdesc
        cmd = policy.UpdateQoSPolicy(test_cli20.MyApp(sys.stdout),
                                     None)
        self._test_update_resource(self.res, cmd, 'myid',
                                   ['myid', '--description', 'newdesc'],
                                   {'description': 'newdesc', },
                                   cmd_resource=self.cmd_res)

    def test_update_policy_to_shared(self):
        # policy-update myid --shared
        cmd = policy.UpdateQoSPolicy(test_cli20.MyApp(sys.stdout),
                                     None)
        self._test_update_resource(self.res, cmd, 'myid',
                                   ['myid', '--shared'],
                                   {'shared': True, },
                                   cmd_resource=self.cmd_res)

    def test_update_policy_to_no_shared(self):
        # policy-update myid --no-shared
        cmd = policy.UpdateQoSPolicy(test_cli20.MyApp(sys.stdout),
                                     None)
        self._test_update_resource(self.res, cmd, 'myid',
                                   ['myid', '--no-shared'],
                                   {'shared': False, },
                                   cmd_resource=self.cmd_res)

    def test_update_policy_to_shared_no_shared_together(self):
        # policy-update myid --shared --no-shared
        cmd = policy.UpdateQoSPolicy(test_cli20.MyApp(sys.stdout),
                                     None)
        self.assertRaises(
            SystemExit,
            self._test_update_resource,
            self.res, cmd, 'myid',
            ['myid', '--shared', '--no-shared'], {},
            cmd_resource=self.cmd_res
        )

    def test_list_policies(self):
        # qos-policy-list.
        cmd = policy.ListQoSPolicy(test_cli20.MyApp(sys.stdout),
                                   None)
        self._test_list_resources(self.ress, cmd, True,
                                  cmd_resources=self.cmd_ress)

    def test_list_policies_pagination(self):
        # qos-policy-list for pagination.
        cmd = policy.ListQoSPolicy(test_cli20.MyApp(sys.stdout),
                                   None)
        self._test_list_resources_with_pagination(self.ress, cmd,
                                                  cmd_resources=self.cmd_ress)

    def test_list_policies_sort(self):
        # sorted list: qos-policy-list --sort-key name --sort-key id
        # --sort-key asc --sort-key desc
        cmd = policy.ListQoSPolicy(test_cli20.MyApp(sys.stdout),
                                   None)
        self._test_list_resources(self.ress, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"],
                                  cmd_resources=self.cmd_ress)

    def test_list_policies_limit(self):
        # size (1000) limited list: qos-policy-list -P.
        cmd = policy.ListQoSPolicy(test_cli20.MyApp(sys.stdout),
                                   None)
        self._test_list_resources(self.ress, cmd, page_size=1000,
                                  cmd_resources=self.cmd_ress)

    def test_show_policy_id(self):
        # qos-policy-show test_id.
        cmd = policy.ShowQoSPolicy(test_cli20.MyApp(sys.stdout),
                                   None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(self.res, cmd, self.test_id, args,
                                 ['id'], cmd_resource=self.cmd_res)

    def test_show_policy_name(self):
        # qos-policy-show.
        cmd = policy.ShowQoSPolicy(test_cli20.MyApp(sys.stdout),
                                   None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(self.res, cmd, self.test_id,
                                 args, ['id', 'name'],
                                 cmd_resource=self.cmd_res)

    def test_delete_policy(self):
        # qos-policy-delete my-id.
        cmd = policy.DeleteQoSPolicy(test_cli20.MyApp(sys.stdout),
                                     None)
        my_id = 'myid1'
        args = [my_id]
        self._test_delete_resource(self.res, cmd, my_id, args,
                                   cmd_resource=self.cmd_res)



# Copyright 2013 Mirantis Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.lb import vip
from neutronclient.tests.unit import test_cli20


class CLITestV20LbVipJSON(test_cli20.CLITestV20Base):
    def setUp(self):
        super(CLITestV20LbVipJSON, self).setUp(plurals={'tags': 'tag'})

    def test_create_vip_with_mandatory_params(self):
        # lb-vip-create with all mandatory params.
        resource = 'vip'
        cmd = vip.CreateVip(test_cli20.MyApp(sys.stdout), None)
        pool_id = 'my-pool-id'
        name = 'my-name'
        subnet_id = 'subnet-id'
        protocol_port = '1000'
        protocol = 'TCP'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--name', name,
                '--protocol-port', protocol_port,
                '--protocol', protocol,
                '--subnet-id', subnet_id,
                '--tenant-id', tenant_id,
                pool_id]
        position_names = ['pool_id', 'name', 'protocol_port', 'protocol',
                          'subnet_id', 'tenant_id']
        position_values = [pool_id, name, protocol_port, protocol,
                           subnet_id, tenant_id]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   admin_state_up=True)

    def test_create_vip_with_all_params(self):
        # lb-vip-create with all params.
        resource = 'vip'
        cmd = vip.CreateVip(test_cli20.MyApp(sys.stdout), None)
        pool_id = 'my-pool-id'
        name = 'my-name'
        description = 'my-desc'
        address = '10.0.0.2'
        admin_state = False
        connection_limit = '1000'
        subnet_id = 'subnet-id'
        protocol_port = '80'
        protocol = 'TCP'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--name', name,
                '--description', description,
                '--address', address,
                '--admin-state-down',
                '--connection-limit', connection_limit,
                '--protocol-port', protocol_port,
                '--protocol', protocol,
                '--subnet-id', subnet_id,
                '--tenant-id', tenant_id,
                pool_id]
        position_names = ['pool_id', 'name', 'description', 'address',
                          'admin_state_up', 'connection_limit',
                          'protocol_port', 'protocol', 'subnet_id',
                          'tenant_id']
        position_values = [pool_id, name, description, address,
                           admin_state, connection_limit, protocol_port,
                           protocol, subnet_id,
                           tenant_id]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def test_create_vip_with_session_persistence_params(self):
        # lb-vip-create with mandatory and session-persistence params.
        resource = 'vip'
        cmd = vip.CreateVip(test_cli20.MyApp(sys.stdout), None)
        pool_id = 'my-pool-id'
        name = 'my-name'
        subnet_id = 'subnet-id'
        protocol_port = '1000'
        protocol = 'TCP'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--name', name,
                '--protocol-port', protocol_port,
                '--protocol', protocol,
                '--subnet-id', subnet_id,
                '--tenant-id', tenant_id,
                pool_id,
                '--session-persistence', 'type=dict',
                'type=cookie,cookie_name=pie',
                '--optional-param', 'any']
        position_names = ['pool_id', 'name', 'protocol_port', 'protocol',
                          'subnet_id', 'tenant_id', 'optional_param']
        position_values = [pool_id, name, protocol_port, protocol,
                           subnet_id, tenant_id, 'any']
        extra_body = {
            'session_persistence': {
                'type': 'cookie',
                'cookie_name': 'pie',
            },
        }
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   admin_state_up=True, extra_body=extra_body)

    def test_list_vips(self):
        # lb-vip-list.
        resources = "vips"
        cmd = vip.ListVip(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_vips_pagination(self):
        # lb-vip-list.
        resources = "vips"
        cmd = vip.ListVip(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_vips_sort(self):
        # lb-vip-list --sort-key name --sort-key id --sort-key asc
        # --sort-key desc
        resources = "vips"
        cmd = vip.ListVip(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_vips_limit(self):
        # lb-vip-list -P.
        resources = "vips"
        cmd = vip.ListVip(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_vip_id(self):
        # lb-vip-show test_id.
        resource = 'vip'
        cmd = vip.ShowVip(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_vip_id_name(self):
        # lb-vip-show.
        resource = 'vip'
        cmd = vip.ShowVip(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_vip(self):
        # lb-vip-update  myid --name myname --tags a b.
        resource = 'vip'
        cmd = vip.UpdateVip(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--tags', 'a', 'b'],
                                   {'name': 'myname', 'tags': ['a', 'b'], })

    def test_update_vip_with_session_persistence(self):
        resource = 'vip'
        cmd = vip.UpdateVip(test_cli20.MyApp(sys.stdout), None)
        body = {
            'session_persistence': {
                'type': 'source',
            },
        }
        args = ['myid', '--session-persistence', 'type=dict',
                'type=source']
        self._test_update_resource(resource, cmd, 'myid', args, body)

    def test_update_vip_with_session_persistence_and_name(self):
        resource = 'vip'
        cmd = vip.UpdateVip(test_cli20.MyApp(sys.stdout), None)
        body = {
            'name': 'newname',
            'session_persistence': {
                'type': 'cookie',
                'cookie_name': 'pie',
            },
        }
        args = ['myid', '--name', 'newname',
                '--session-persistence', 'type=dict',
                'type=cookie,cookie_name=pie']
        self._test_update_resource(resource, cmd, 'myid', args, body)

    def test_delete_vip(self):
        # lb-vip-delete my-id.
        resource = 'vip'
        cmd = vip.DeleteVip(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)



# Copyright 2014 Blue Box Group, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.common import exceptions
from neutronclient.neutron.v2_0.lb.v2 import pool
from neutronclient.tests.unit import test_cli20


class CLITestV20LbPoolJSON(test_cli20.CLITestV20Base):

    def test_create_pool_with_listener(self):
        # lbaas-pool-create with listener
        resource = 'pool'
        cmd_resource = 'lbaas_pool'
        cmd = pool.CreatePool(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        lb_algorithm = 'ROUND_ROBIN'
        listener = 'listener'
        protocol = 'TCP'
        args = ['--lb-algorithm', lb_algorithm, '--protocol', protocol,
                '--listener', listener]
        position_names = ['admin_state_up', 'lb_algorithm', 'protocol',
                          'listener_id']
        position_values = [True, lb_algorithm, protocol, listener]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource)

    def test_create_pool_with_loadbalancer_no_listener(self):
        """lbaas-pool-create with loadbalancer, no listener."""
        resource = 'pool'
        cmd_resource = 'lbaas_pool'
        cmd = pool.CreatePool(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        lb_algorithm = 'ROUND_ROBIN'
        loadbalancer = 'loadbalancer'
        protocol = 'TCP'
        args = ['--lb-algorithm', lb_algorithm, '--protocol', protocol,
                '--loadbalancer', loadbalancer]
        position_names = ['admin_state_up', 'lb_algorithm', 'protocol',
                          'loadbalancer_id']
        position_values = [True, lb_algorithm, protocol, loadbalancer]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource)

    def test_create_pool_with_no_listener_or_loadbalancer(self):
        """lbaas-pool-create with no listener or loadbalancer."""
        resource = 'pool'
        cmd_resource = 'lbaas_pool'
        cmd = pool.CreatePool(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        lb_algorithm = 'ROUND_ROBIN'
        protocol = 'TCP'
        args = ['--lb-algorithm', lb_algorithm, '--protocol', protocol]
        position_names = ['admin_state_up', 'lb_algorithm', 'protocol']
        position_values = [True, lb_algorithm, protocol]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource,
                                   no_api_call=True,
                                   expected_exception=exceptions.CommandError)

    def test_create_pool_with_all_params(self):
        # lbaas-pool-create with all params set.
        resource = 'pool'
        cmd_resource = 'lbaas_pool'
        cmd = pool.CreatePool(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        lb_algorithm = 'ROUND_ROBIN'
        listener = 'listener'
        loadbalancer = 'loadbalancer'
        protocol = 'TCP'
        description = 'description'
        session_persistence_str = 'type=APP_COOKIE,cookie_name=1234'
        session_persistence = {'type': 'APP_COOKIE',
                               'cookie_name': '1234'}
        name = 'my-pool'
        args = ['--lb-algorithm', lb_algorithm, '--protocol', protocol,
                '--description', description, '--session-persistence',
                session_persistence_str, '--admin-state-down', '--name', name,
                '--listener', listener, '--loadbalancer', loadbalancer]
        position_names = ['lb_algorithm', 'protocol', 'description',
                          'session_persistence', 'admin_state_up', 'name',
                          'listener_id', 'loadbalancer_id']
        position_values = [lb_algorithm, protocol, description,
                           session_persistence, False, name, listener,
                           loadbalancer]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource)

    def test_list_pools(self):
        # lbaas-pool-list.
        resources = 'pools'
        cmd_resources = 'lbaas_pools'
        cmd = pool.ListPool(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True,
                                  cmd_resources=cmd_resources)

    def test_list_pools_pagination(self):
        # lbaas-pool-list with pagination.
        resources = 'pools'
        cmd_resources = 'lbaas_pools'
        cmd = pool.ListPool(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd,
                                                  cmd_resources=cmd_resources)

    def test_list_pools_sort(self):
        # lbaas-pool-list --sort-key id --sort-key asc.
        resources = 'pools'
        cmd_resources = 'lbaas_pools'
        cmd = pool.ListPool(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True,
                                  cmd_resources=cmd_resources)

    def test_list_pools_limit(self):
        # lbaas-pool-list -P.
        resources = 'pools'
        cmd_resources = 'lbaas_pools'
        cmd = pool.ListPool(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000,
                                  cmd_resources=cmd_resources)

    def test_show_pool_id(self):
        # lbaas-pool-show test_id.
        resource = 'pool'
        cmd_resource = 'lbaas_pool'
        cmd = pool.ShowPool(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'],
                                 cmd_resource=cmd_resource)

    def test_show_pool_id_name(self):
        # lbaas-pool-show.
        resource = 'pool'
        cmd_resource = 'lbaas_pool'
        cmd = pool.ShowPool(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'],
                                 cmd_resource=cmd_resource)

    def test_update_pool(self):
        # lbaas-pool-update myid --name newname --description SuperPool
        # --lb-algorithm SOURCE_IP --admin-state-up
        # --session-persistence type=dict,type=HTTP_COOKIE,cookie_name=pie

        resource = 'pool'
        cmd_resource = 'lbaas_pool'
        cmd = pool.UpdatePool(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', '--name', 'newname',
                '--description', 'SuperPool', '--lb-algorithm', "SOURCE_IP",
                '--admin-state-up', 'True',
                '--session-persistence', 'type=dict,'
                'type=HTTP_COOKIE,cookie_name=pie']
        body = {'name': 'newname',
                "description": "SuperPool",
                "lb_algorithm": "SOURCE_IP",
                "admin_state_up": 'True',
                'session_persistence': {
                    'type': 'HTTP_COOKIE',
                    'cookie_name': 'pie',
                }, }
        self._test_update_resource(resource, cmd, 'myid', args, body,
                                   cmd_resource=cmd_resource)
        # lbaas-pool-update myid --name Name
        # --no-session-persistence

        resource = 'pool'
        cmd_resource = 'lbaas_pool'
        cmd = pool.UpdatePool(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', '--name', 'Name', '--no-session-persistence']
        body = {'name': "Name",
                "session_persistence": None, }
        self._test_update_resource(resource, cmd, 'myid', args, body,
                                   cmd_resource=cmd_resource)

    def test_delete_pool(self):
        # lbaas-pool-delete my-id.
        resource = 'pool'
        cmd_resource = 'lbaas_pool'
        cmd = pool.DeletePool(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args,
                                   cmd_resource=cmd_resource)



# Copyright (c) 2016 Juniper Networks Inc.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

BGPVPN = 'bgpvpn'
BGPVPNS = '%ss' % BGPVPN

NETWORK_RESOURCE_NAME = 'network'
NETWORK_ASSOC = '%s_association' % NETWORK_RESOURCE_NAME
NETWORK_ASSOCS = '%ss' % NETWORK_ASSOC

ROUTER_RESOURCE_NAME = 'router'
ROUTER_ASSOC = '%s_association' % ROUTER_RESOURCE_NAME
ROUTER_ASSOCS = '%ss' % ROUTER_ASSOC

PORT_RESOURCE_NAME = 'port'
PORT_ASSOC = '%s_association' % PORT_RESOURCE_NAME
PORT_ASSOCS = '%ss' % PORT_ASSOC



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from osc_lib.command import command
from osc_lib import utils

from neutronclient._i18n import _
from neutronclient.osc.v2.dynamic_routing import constants


def _format_alive_state(item):
    return ':-)' if item else 'XXX'


_formatters = {
    'alive': _format_alive_state
}


def add_common_args(parser):
    parser.add_argument('dragent_id',
                        metavar='<agent-id>',
                        help=_("ID of the dynamic routing agent"))
    parser.add_argument('bgp_speaker',
                        metavar='<bgp-speaker>',
                        help=_("ID or name of the BGP speaker"))


class AddBgpSpeakerToDRAgent(command.Command):
    """Add a BGP speaker to a dynamic routing agent"""

    def get_parser(self, prog_name):
        parser = super(AddBgpSpeakerToDRAgent, self).get_parser(prog_name)
        add_common_args(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        speaker_id = client.find_resource(constants.BGP_SPEAKER,
                                          parsed_args.bgp_speaker)['id']
        client.add_bgp_speaker_to_dragent(
            parsed_args.dragent_id, {'bgp_speaker_id': speaker_id})


class RemoveBgpSpeakerFromDRAgent(command.Command):
    """Removes a BGP speaker from a dynamic routing agent"""

    def get_parser(self, prog_name):
        parser = super(RemoveBgpSpeakerFromDRAgent, self).get_parser(
            prog_name)
        add_common_args(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        speaker_id = client.find_resource(constants.BGP_SPEAKER,
                                          parsed_args.bgp_speaker)['id']
        client.remove_bgp_speaker_from_dragent(parsed_args.dragent_id,
                                               speaker_id)


class ListDRAgentsHostingBgpSpeaker(command.Lister):
    """(Deprecated) List dynamic routing agents hosting a BGP speaker

       (Use "bgp dragent list" instead)
    """

    resource = 'agent'
    list_columns = ['id', 'host', 'admin_state_up', 'alive']
    unknown_parts_flag = False

    def get_parser(self, prog_name):
        self.log.warning("The 'openstack bgp speaker show dragents' CLI is "
                         "deprecated and will be removed in the future. Use "
                         "'openstack bgp dragent list' CLI instead.")
        parser = super(ListDRAgentsHostingBgpSpeaker,
                       self).get_parser(prog_name)
        parser.add_argument('bgp_speaker',
                            metavar='<bgp-speaker>',
                            help=_("List dynamic routing agents hosting a "
                                   "BGP speaker (name or ID)"))
        return parser

    def take_action(self, parsed_args):
        search_opts = {}
        client = self.app.client_manager.neutronclient
        speaker_id = client.find_resource(constants.BGP_SPEAKER,
                                          parsed_args.bgp_speaker)['id']
        search_opts['bgp_speaker'] = speaker_id
        data = client.list_dragents_hosting_bgp_speaker(**search_opts)
        headers = ('ID', 'Host', 'State', 'Alive')
        columns = ('id', 'host', 'admin_state_up', 'alive')
        return (headers,
                (utils.get_dict_properties(
                    s, columns, formatters=_formatters,
                ) for s in data['agents']))


class ListDRAgent(command.Lister):
    """List dynamic routing agents"""

    resource = 'agent'
    list_columns = ['id', 'host', 'admin_state_up', 'alive']
    unknown_parts_flag = False

    def get_parser(self, prog_name):
        parser = super(ListDRAgent,
                       self).get_parser(prog_name)
        parser.add_argument('--bgp-speaker',
                            metavar='<bgp-speaker>',
                            help=_("List dynamic routing agents hosting a "
                                   "BGP speaker (name or ID)"))
        return parser

    def take_action(self, parsed_args):
        search_opts = {}
        client = self.app.client_manager.neutronclient
        if parsed_args.bgp_speaker is not None:
            search_opts = {}
            speaker_id = client.find_resource(constants.BGP_SPEAKER,
                                              parsed_args.bgp_speaker)['id']
            search_opts['bgp_speaker'] = speaker_id
            data = client.list_dragents_hosting_bgp_speaker(**search_opts)
        else:
            attrs = {'agent_type': 'BGP dynamic routing agent'}
            data = client.list_agents(**attrs)
        headers = ('ID', 'Host', 'State', 'Alive')
        columns = ('id', 'host', 'admin_state_up', 'alive')
        return (headers,
                (utils.get_dict_properties(
                    s, columns, formatters=_formatters,
                ) for s in data['agents']))



# Copyright (c) 2018 Orange SA.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy
import operator
from unittest import mock

from osc_lib.tests.utils import ParserException
from osc_lib import utils as osc_utils
from osc_lib.utils import columns as column_util

from neutronclient.tests.unit.osc.v2.networking_bgpvpn import fakes


columns_short = tuple(col for col, _, listing_mode
                      in fakes.BgpvpnFakeRouterAssoc._attr_map
                      if listing_mode in (column_util.LIST_BOTH,
                                          column_util.LIST_SHORT_ONLY))
columns_long = tuple(col for col, _, listing_mode
                     in fakes.BgpvpnFakeRouterAssoc._attr_map
                     if listing_mode in (column_util.LIST_BOTH,
                                         column_util.LIST_LONG_ONLY))
headers_short = tuple(head for _, head, listing_mode
                      in fakes.BgpvpnFakeRouterAssoc._attr_map
                      if listing_mode in (column_util.LIST_BOTH,
                                          column_util.LIST_SHORT_ONLY))
headers_long = tuple(head for _, head, listing_mode
                     in fakes.BgpvpnFakeRouterAssoc._attr_map
                     if listing_mode in (column_util.LIST_BOTH,
                                         column_util.LIST_LONG_ONLY))
sorted_attr_map = sorted(fakes.BgpvpnFakeRouterAssoc._attr_map,
                         key=operator.itemgetter(1))
sorted_columns = tuple(col for col, _, _ in sorted_attr_map)
sorted_headers = tuple(head for _, head, _ in sorted_attr_map)


def _get_data(attrs, columns=sorted_columns):
    return osc_utils.get_dict_properties(
        attrs, columns, formatters=fakes.BgpvpnFakeAssoc._formatters)


class TestCreateRouterAssoc(fakes.TestNeutronClientBgpvpn):
    def setUp(self):
        super(TestCreateRouterAssoc, self).setUp()
        self.cmd = fakes.CreateBgpvpnFakeRouterAssoc(self.app, self.namespace)
        self.fake_bgpvpn = fakes.FakeBgpvpn.create_one_bgpvpn()
        self.fake_router = fakes.FakeResource.create_one_resource()

    def _build_args(self, param=None):
        arglist_base = [
            self.fake_bgpvpn['id'],
            self.fake_router['id'],
            '--project', self.fake_bgpvpn['tenant_id']
        ]
        if param is not None:
            if isinstance(param, list):
                arglist_base.extend(param)
            else:
                arglist_base.append(param)
        return arglist_base

    def _build_verify_list(self, param=None):
        verifylist = [
            ('bgpvpn', self.fake_bgpvpn['id']),
            ('resource', self.fake_router['id']),
            ('project', self.fake_bgpvpn['tenant_id'])
        ]
        if param is not None:
            verifylist.append(param)
        return verifylist

    def _exec_create_router_association(
            self, fake_res_assoc, arglist, verifylist):

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cols, data = self.cmd.take_action(parsed_args)

        fake_res_assoc_call = copy.deepcopy(fake_res_assoc)
        fake_res_assoc_call.pop('id')

        self.neutronclient.create_bgpvpn_fake_resource_assoc.\
            assert_called_once_with(
                self.fake_bgpvpn['id'],
                {fakes.BgpvpnFakeRouterAssoc._resource: fake_res_assoc_call})
        return cols, data

    def test_create_router_association(self):
        fake_res_assoc = fakes.FakeResAssoc.create_one_resource_association(
            self.fake_router)

        self.neutronclient.create_bgpvpn_fake_resource_assoc = mock.Mock(
            return_value={
                fakes.BgpvpnFakeRouterAssoc._resource: fake_res_assoc,
                'advertise_extra_routes': True})

        arglist = self._build_args()
        # advertise_extra_routes will be False since none
        # of the mutually exclusive args present
        verifylist = self._build_verify_list(('advertise_extra_routes', False))

        self._exec_create_router_association(
            fake_res_assoc, arglist, verifylist)

    def test_create_router_association_advertise(self):
        fake_res_assoc = fakes.FakeResAssoc.create_one_resource_association(
            self.fake_router,
            {'advertise_extra_routes': True})

        self.neutronclient.create_bgpvpn_fake_resource_assoc = mock.Mock(
            return_value={
                fakes.BgpvpnFakeRouterAssoc._resource: fake_res_assoc})

        arglist = self._build_args('--advertise_extra_routes')
        verifylist = self._build_verify_list(('advertise_extra_routes', True))

        cols, data = self._exec_create_router_association(
            fake_res_assoc, arglist, verifylist)
        self.assertEqual(sorted_headers, cols)
        self.assertEqual(_get_data(fake_res_assoc), data)

    def test_create_router_association_no_advertise(self):
        fake_res_assoc = fakes.FakeResAssoc.create_one_resource_association(
            self.fake_router,
            {'advertise_extra_routes': False})

        self.neutronclient.create_bgpvpn_fake_resource_assoc = mock.Mock(
            return_value={
                fakes.BgpvpnFakeRouterAssoc._resource: fake_res_assoc})

        arglist = self._build_args('--no-advertise_extra_routes')
        verifylist = self._build_verify_list(('advertise_extra_routes', False))

        cols, data = self._exec_create_router_association(
            fake_res_assoc, arglist, verifylist)
        self.assertEqual(sorted_headers, cols)
        self.assertEqual(_get_data(fake_res_assoc), data)

    def test_create_router_association_advertise_fault(self):
        arglist = self._build_args(
            ['--advertise_extra_routes', '--no-advertise_extra_routes'])

        try:
            self._exec_create_router_association(None, arglist, None)
        except ParserException as e:
            self.assertEqual(format(e), 'Argument parse failed')

    def test_router_association_unknown_arg(self):
        arglist = self._build_args('--unknown arg')

        try:
            self._exec_create_router_association(None, arglist, None)
        except ParserException as e:
            self.assertEqual(format(e), 'Argument parse failed')


class TestSetRouterAssoc(fakes.TestNeutronClientBgpvpn):

    def setUp(self):
        super(TestSetRouterAssoc, self).setUp()
        self.cmd = fakes.SetBgpvpnFakeRouterAssoc(self.app, self.namespace)
        self.fake_bgpvpn = fakes.FakeBgpvpn.create_one_bgpvpn()
        self.fake_router = fakes.FakeResource.create_one_resource()

    def _build_args(self, fake_res_assoc, param=None):
        arglist_base = [
            fake_res_assoc['id'],
            self.fake_bgpvpn['id']
        ]
        if param is not None:
            if isinstance(param, list):
                arglist_base.extend(param)
            else:
                arglist_base.append(param)
        return arglist_base

    def _build_verify_list(self, fake_res_assoc, param=None):
        verifylist = [
            ('resource_association_id', fake_res_assoc['id']),
            ('bgpvpn', self.fake_bgpvpn['id'])
        ]
        if param is not None:
            verifylist.append(param)
        return verifylist

    def test_set_router_association_no_advertise(self):
        fake_res_assoc = fakes.FakeResAssoc.create_one_resource_association(
            self.fake_router,
            {'advertise_extra_routes': True})
        self.neutronclient.update_bgpvpn_fake_resource_assoc = mock.Mock()

        arglist = self._build_args(
            fake_res_assoc,
            '--no-advertise_extra_routes')
        verifylist = [
            ('resource_association_id', fake_res_assoc['id']),
            ('bgpvpn', self.fake_bgpvpn['id']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        fake_res_assoc_call = copy.deepcopy(fake_res_assoc)
        fake_res_assoc_call.pop('id')

        self.neutronclient.update_bgpvpn_fake_resource_assoc.\
            assert_called_once_with(
                self.fake_bgpvpn['id'],
                fake_res_assoc['id'],
                {
                    fakes.BgpvpnFakeRouterAssoc._resource: {
                        'advertise_extra_routes': False
                    }
                })
        self.assertIsNone(result)

    def test_set_router_association_advertise(self):
        fake_res_assoc = fakes.FakeResAssoc.create_one_resource_association(
            self.fake_router,
            {'advertise_extra_routes': False})
        self.neutronclient.update_bgpvpn_fake_resource_assoc = mock.Mock()

        arglist = self._build_args(
            fake_res_assoc,
            '--advertise_extra_routes')
        verifylist = [
            ('resource_association_id', fake_res_assoc['id']),
            ('bgpvpn', self.fake_bgpvpn['id']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        fake_res_assoc_call = copy.deepcopy(fake_res_assoc)
        fake_res_assoc_call.pop('id')

        self.neutronclient.update_bgpvpn_fake_resource_assoc.\
            assert_called_once_with(
                self.fake_bgpvpn['id'],
                fake_res_assoc['id'],
                {
                    fakes.BgpvpnFakeRouterAssoc._resource: {
                        'advertise_extra_routes': True
                    }
                })
        self.assertIsNone(result)


class TestShowRouterAssoc(fakes.TestNeutronClientBgpvpn):
    def setUp(self):
        super(TestShowRouterAssoc, self).setUp()
        self.cmd = fakes.ShowBgpvpnFakeRouterAssoc(self.app, self.namespace)

    def test_show_router_association(self):
        fake_bgpvpn = fakes.FakeBgpvpn.create_one_bgpvpn()
        fake_res = fakes.FakeResource.create_one_resource()
        fake_res_assoc = fakes.FakeResAssoc.create_one_resource_association(
            fake_res,
            {'advertise_extra_routes': True})
        self.neutronclient.show_bgpvpn_fake_resource_assoc = mock.Mock(
            return_value={fakes.BgpvpnFakeAssoc._resource: fake_res_assoc})
        arglist = [
            fake_res_assoc['id'],
            fake_bgpvpn['id'],
        ]
        verifylist = [
            ('resource_association_id', fake_res_assoc['id']),
            ('bgpvpn', fake_bgpvpn['id']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        headers, data = self.cmd.take_action(parsed_args)

        self.neutronclient.show_bgpvpn_fake_resource_assoc.\
            assert_called_once_with(fake_bgpvpn['id'], fake_res_assoc['id'])
        self.assertEqual(sorted_headers, headers)
        self.assertEqual(data, _get_data(fake_res_assoc))



# Copyright 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20


class ListFlavor(neutronV20.ListCommand):
    """List Neutron service flavors."""

    resource = 'flavor'
    list_columns = ['id', 'name', 'service_type', 'enabled']
    pagination_support = True
    sorting_support = True


class ShowFlavor(neutronV20.ShowCommand):
    """Show information about a given Neutron service flavor."""

    resource = 'flavor'


class CreateFlavor(neutronV20.CreateCommand):
    """Create a Neutron service flavor."""

    resource = 'flavor'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name',
            metavar='NAME',
            help=_('Name for the flavor.'))
        parser.add_argument(
            'service_type',
            metavar='SERVICE_TYPE',
            help=_('Service type to which the flavor applies to: e.g. VPN. '
                   '(See service-provider-list for loaded examples.)'))
        parser.add_argument(
            '--description',
            help=_('Description for the flavor.'))
        utils.add_boolean_argument(
            parser,
            '--enabled',
            default=argparse.SUPPRESS,
            help=_('Sets enabled flag.'))

    def args2body(self, parsed_args):
        body = {}
        neutronV20.update_dict(parsed_args, body,
                               ['name', 'description', 'service_type',
                                'enabled'])
        return {self.resource: body}


class DeleteFlavor(neutronV20.DeleteCommand):
    """Delete a given Neutron service flavor."""

    resource = 'flavor'


class UpdateFlavor(neutronV20.UpdateCommand):
    """Update a Neutron service flavor."""

    resource = 'flavor'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name',
            help=_('Name for the flavor.'))
        parser.add_argument(
            '--description',
            help=_('Description for the flavor.'))
        utils.add_boolean_argument(
            parser,
            '--enabled',
            default=argparse.SUPPRESS,
            help=_('Sets enabled flag.'))

    def args2body(self, parsed_args):
        body = {}
        neutronV20.update_dict(parsed_args, body,
                               ['name', 'description', 'enabled'])
        return {self.resource: body}


class AssociateFlavor(neutronV20.NeutronCommand):
    """Associate a Neutron service flavor with a flavor profile."""

    resource = 'flavor'

    def get_parser(self, prog_name):
        parser = super(AssociateFlavor, self).get_parser(prog_name)
        parser.add_argument(
            'flavor',
            metavar='FLAVOR',
            help=_('ID or name of the flavor to associate.'))
        parser.add_argument(
            'flavor_profile',
            metavar='FLAVOR_PROFILE',
            help=_('ID of the flavor profile to be associated with the '
                   'flavor.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        flavor_id = neutronV20.find_resourceid_by_name_or_id(
            neutron_client, 'flavor', parsed_args.flavor)
        service_profile_id = neutronV20.find_resourceid_by_id(
            neutron_client, 'service_profile', parsed_args.flavor_profile)
        body = {'service_profile': {'id': service_profile_id}}
        neutron_client.associate_flavor(flavor_id, body)
        print((_('Associated flavor %(flavor)s with '
                 'flavor_profile %(profile)s') %
               {'flavor': parsed_args.flavor,
                'profile': parsed_args.flavor_profile}),
              file=self.app.stdout)


class DisassociateFlavor(neutronV20.NeutronCommand):
    """Disassociate a Neutron service flavor from a flavor profile."""

    resource = 'flavor'

    def get_parser(self, prog_name):
        parser = super(DisassociateFlavor, self).get_parser(prog_name)
        parser.add_argument(
            'flavor',
            metavar='FLAVOR',
            help=_('ID or name of the flavor to be disassociated.'))
        parser.add_argument(
            'flavor_profile',
            metavar='FLAVOR_PROFILE',
            help=_('ID of the flavor profile to be disassociated from the '
                   'flavor.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        flavor_id = neutronV20.find_resourceid_by_name_or_id(
            neutron_client, 'flavor', parsed_args.flavor)
        service_profile_id = neutronV20.find_resourceid_by_id(
            neutron_client, 'service_profile', parsed_args.flavor_profile)
        neutron_client.disassociate_flavor(flavor_id, service_profile_id)
        print((_('Disassociated flavor %(flavor)s from '
                 'flavor_profile %(profile)s') %
               {'flavor': parsed_args.flavor,
                'profile': parsed_args.flavor_profile}),
              file=self.app.stdout)
